import os
import sys
from gettext import translation

from turbogears import config
from turbogears.util import get_package_name, request_available
from turbogears.i18n.utils import get_locale
from turbojson.jsonify import jsonify

_catalogs = {}

def get_locale_dir():
    localedir = config.get("i18n.locale_dir")
    if not localedir:
        package = get_package_name()
        if package:
            localedir = os.path.join(os.path.dirname(
                sys.modules[package].__path__[0]), "locales")
    return localedir

def is_locale_supported(locale, domain=None):
    """Check if [domain].mo file exists for this language."""
    if not domain:
        domain = config.get("i18n.domain", "messages")

    localedir = get_locale_dir()
    return localedir and os.path.exists(os.path.join(
        localedir, locale, "LC_MESSAGES", "%s.mo" % domain))

def get_catalog(locale, domain = None):
    """Return translations for given locale."""
    if not domain:
        domain = config.get("i18n.domain", "messages")

    catalog = _catalogs.get(domain)

    if not catalog:
        catalog = _catalogs[domain] = {}

    messages = catalog.get(locale)
    if not messages:
        localedir = get_locale_dir()
        messages = catalog[locale] = translation(domain=domain,
            localedir=localedir, languages=[locale])

    return messages

def plain_gettext(key, locale=None, domain=None):
    """Get the gettext value for key.

    Added to builtins as '_'. Returns Unicode string.

    @param key: text to be translated
    @param locale: locale code to be used.
        If locale is None, gets the value provided by get_locale.

    """
    gettext_func = config.get("i18n.gettext", tg_gettext)
    return gettext_func(key, locale, domain)

def tg_gettext(key, locale=None, domain=None):
    """Get the gettext value for key.

    Added to builtins as '_'. Returns Unicode string.

    @param key: text to be translated
    @param locale: locale code to be used.
        If locale is None, gets the value provided by get_locale.

    """
    if locale is None:
        locale = get_locale()
    if not is_locale_supported(locale):
        locale = locale[:2]
    if key == '':
        return '' # special case
    try:
        return get_catalog(locale, domain).ugettext(key)
    except KeyError:
        return key
    except IOError:
        return key

def plain_ngettext(key1, key2, num, locale=None):
    """Translate two possible texts based on whether num is greater than 1.

    @param key1: text if num==1
    @param key2: text if num!=1
    @param num: a number
    @type num: integer
    @locale: locale code to be used.
        If locale is None, gets the value provided by get_locale.

    """
    if num==1:
        return plain_gettext(key1, locale)
    else:
        return plain_gettext(key2, locale)

class lazystring(object):
    """Has a number of lazily evaluated functions replicating a string.

    Just override the eval() method to produce the actual value.

    """

    def __init__(self, func, *args, **kw):
        self.func = func
        self.args = args
        self.kw = kw

    def eval(self):
        return self.func(*self.args, **self.kw)

    def __unicode__(self):
        return unicode(self.eval())

    def __str__(self):
        return str(self.eval())

    def __mod__(self, other):
        return self.eval() % other

    def __cmp__(self, other):
        return cmp(self.eval(), other)

    def __eq__(self, other):
        return self.eval() == other

    def __deepcopy__(self, memo):
        return self

def lazify(func):
    def newfunc(*args, **kw):
        lazystr = lazystring(func, *args, **kw)
        return lazystr
    return newfunc

[jsonify.when("isinstance(obj, lazystring)")]
def jsonify_lazystring(obj):
    return unicode(obj)

lazy_gettext = lazify(plain_gettext)
lazy_ngettext = lazify(plain_ngettext)

def gettext(key, locale=None, domain=None):
    """Get the gettext value for key.

    Added to builtins as '_'. Returns Unicode string.

    @param key: text to be translated
    @param locale: locale code to be used.
        If locale is None, gets the value provided by get_locale.

    """
    if request_available():
        return plain_gettext(key, locale, domain)
    else:
        return lazy_gettext(key, locale, domain)

def ngettext(key1, key2, num, locale=None):
    """Translate two possible texts based on whether num is greater than 1.

    @param key1: text if num==1
    @param key2: text if num!=1
    @param num: a number
    @type num: integer
    @param locale: locale code to be used.
        If locale is None, gets the value provided by get_locale.

    """
    if request_available():
        return plain_ngettext(key1, key2, num, locale)
    else:
        return lazy_ngettext(key1, key2, num, locale)

def install():
    """Add the gettext function to __builtins__ as '_'."""
    __builtins__["_"] = gettext



"""Graphical user interface for SQLObject's model design"""

import os
import string
import shutil

import pkg_resources

import cherrypy

try:
    import sqlobject
except ImportError:
    sqlobject = None

import turbogears
from turbogears import controllers, expose
from turbogears.util import get_model, get_package_name, get_project_name


model_template = """from datetime import datetime
from turbogears.database import PackageHub
from sqlobject import *
from turbogears import identity

hub = PackageHub("%s")
__connection__ = hub

%s
"""

session_file_name = 'model_designer.tmp'


def version_file(file_spec, vtype='copy'):
    """By Robin Parmar & Martin Miller from Python Cookbook 2. edition"""

    if os.path.isfile(file_spec):
        # check the 'vtype' parameter
        if vtype not in ('copy', 'rename'):
            raise ValueError, 'Unknown vtype %r' % (vtype,)

        # determine root file name so the extension doesn't get longer and longer...
        n, e = os.path.splitext(file_spec)

        # is e a three-digets integer preceded by a dot?
        if len(e) == 4 and e[1:].isdigit():
            num = 1 + int(e[1:])
            root = n
        else:
            num = 0
            root = file_spec

        # Find next available file version
        for i in xrange(num, 1000):
            new_file = '%s.%03d' % (root, i)
            if not os.path.exists(new_file):
                if vtype == 'copy':
                    shutil.copy(file_spec, new_file)
                else:
                    os.rename(file_spec, new_file)
                return True
        raise RuntimeError, "Can't %s %r, all names taken" % (vtype, file_spec)
    return False


class Designer(controllers.RootController):
    """Designer for SQLObject models.

    Create your classes, define your fields and manage your relations.
    Visualize and generate code for SQLObject models.

    """

    __label__ = "ModelDesigner"
    __version__ = "0.1"
    __author__ = "Ronald Jaramillo"
    __email__ = "ronald@checkandshare.com"
    __copyright__ = "Copyright 2005 Ronald Jaramillo"
    __license__ = "MIT"

    baseTemplate = 'turbogears.toolbox.designer'
    model = None
    icon = "/tg_static/images/designer.png"

    def __init__(self):
        if not sqlobject:
            raise ImportError, ("Cannot run the Model Designer.\n"
                "The SQLObject package is not installed.")

    def set_model(self, model):
        self.model = model

    def get_model_name(self):
        if not get_project_name():
            return False
        return get_model()

    def register_static_directory(self):
        static_directory = pkg_resources.resource_filename(__name__,'static')
        turbogears.config.update({ '/tg_toolbox/designer': {
            'static_filter.on': True,
            'static_filter.dir': static_directory}})


    def column_type(self, column):
        """Given a column representation return the column type."""
        column_type = '%r' % column
        return column_type.split()[0][1:].split('.')[-1]

    def column_default(self, column, column_type):
        try:
            default = column.default
        except AttributeError:
            return ''
        if default == sqlobject.sqlbuilder.NoDefault:
            return ''
        if column_type in ('SOIntCol', 'SOFloatCol', 'SOStringCol',
             'SODecimalCol', 'SOCurrencyCol'):
            return default
        elif column_type == 'SODateTimeCol':
            d = '%s' % default
            return ':'.join(d.split(':')[:-1])
        elif column_type == 'SODateCol':
            d = '%s' % default
            return d.split()[0]
        return ''

    def load_column(self, column_name, column, model_object):
        props = {}
        props['type'] = self.column_type(column)
        props['column_name'] = column_name
        props['name'] = column_name
        props['column_default'] = self.column_default(column, props['type'])
        try:
            props['title'] = getattr(column, 'title') or ''
        except AttributeError:
            props['title'] = ''
        if props['type'] == 'SOStringCol':
            props['length'] = column.length
            props['varchar'] = column.varchar
        if props['type'] == 'SOEnumCol':
            props['enum_values'] = column.enumValues
        if props['type'] == 'SOForeignKey':
            props['join_type'] = 'MultipleJoin'
            props['other_class_name'] = column.foreignKey
            props['other_method_name'] = self.other_method_name(
                column, model_object.__name__)
        if props['type'] in ('SOMultipleJoin','SORelatedJoin'):
            props['other_class_name'] = column.otherClassName
            props['join_type'] = props['type'].replace('SO', '')
            props['other_method_name'] = self.other_method_join_name(
                column, model_object.__name__)

        props['type'] = props['type'].replace('SO','')
        return props

    def other_method_join_name(self, column, model_object_name):
        for col in column.otherClass.sqlmeta.columnList:
            if type(col) == sqlobject.SOForeignKey:
                if col.foreignKey == model_object_name:
                    return col.origName
        return 'id'

    def other_method_name(self, column, model_object_name):
        other_class_name = column.foreignKey
        model_object = getattr(self.get_model_name(), other_class_name)
        for col in model_object.sqlmeta.joins:
            if col.otherClassName == model_object_name:
                return col.joinMethodName
        return 'id'

    def is_inheritable_base_class(self, obj):
        """Check if the object is  a direct subclass of InheritableSQLObject"""
        return 'sqlobject.inheritance.InheritableSQLObject' in str(obj.__bases__)

    def load_columns(self, model_object):
        columns = {}
        # get normal columns
        for column_name in model_object.sqlmeta.columns:
            column = model_object.sqlmeta.columns[column_name]
            origname = column.origName
            if model_object._inheritable and column_name == 'childName':
                continue
            columns[origname] = self.load_column(origname, column, model_object)
        # get join columns
        for column in model_object.sqlmeta.joins:
            columns[column.joinMethodName] = self.load_column(
                column.joinMethodName, column, model_object)
        return columns

    [expose(format='json')]
    def load_current_model(self):
        model = self.get_model_name()
        current_model = {}
        current_model['name'] = get_package_name()
        current_model['models'] = {}
        current_model['ordered_models'] = []

        for m in dir(model):
            if m in ('SQLObject', 'InheritableSQLObject'):
                continue
            model_object = getattr(model, m)
            if isinstance(model_object, type) and issubclass(
                    model_object, sqlobject.SQLObject):
                parent_class = 'SQLObject'
                if model_object._inheritable:
                    parent_class = model_object._parentClass.__name__
                columns = self.load_columns(model_object)
                current_model['ordered_models'].append(m)
                current_model['models'][m] = {
                    'name': m,
                    'parent_class': parent_class,
                    'table_name': model_object.sqlmeta.table,
                    'id_name': model_object.sqlmeta.idName,
                    'columns': columns,
                    'relations': {}}
        return dict(model=current_model)

    def save_session_as_name(self, name):
        # remove non-ascii
        if isinstance(name, unicode):
            name = name.encode('ascii', 'replace')
        # remove punctuation
        name = name.translate(string.maketrans('', ''), string.punctuation)
        # camelcase to remove spaces
        name =  ''.join([x.title() for x in name.split()])
        sessions_directory = pkg_resources.resource_filename(__name__,
            os.path.join('static', 'sessions'))
        for idx in range(100):
            postfix = idx and '_%d' % idx or ''
            test_name = '%s%s.js' % (name, postfix)
            full_name = os.path.join(sessions_directory, test_name)
            if not os.path.exists(full_name):
                return full_name

    [expose(format='json')]
    def save_state(self, state, name=session_file_name):
        if name != session_file_name:
            name = self.save_session_as_name(name)
        if not name:
            return dict(state=False)
        f = open(name,'w')
        f.write(state)
        f.close()
        return dict(state=True)

    def model_path(self):
        return os.path.abspath(
            os.path.join(get_package_name(), 'model.py'))

    def save_model(self, code):
        project_name = get_package_name()
        model_text = model_template % (project_name, code)
        model_path = self.model_path()
        if os.path.exists(model_path):
            version_file(model_path)
            f = open(model_path,'w')
            f.write(model_text)
            f.close()
            return True, 'Saved %s' % model_path
        else:
            return False, 'Failed to save %s' % model_path

    def save_and_create_(self, code, order):
        model_path = os.path.join(get_project_name(),
            'tmp2_model.py')
        model_path = os.path.abspath(model_path)
        open(model_path, 'w').write(str)
        package = __import__(get_package_name(), {}, {},
            ["tmp2_model"])
        package.tmp2_model.Persona.createTable(ifNotExists=True)
        model = self.get_model_name()
        model.Persona = package.tmp2_model.Persona
        # pk = package.model
        # import tmp_model as pk
        # pk.Persona.createTable(ifNotExists=True)
        return dict(status='Tables ok')

    [expose(format='json')]
    def save_and_create(self, code, order):
        status_code, status = self.save_model(code)
        if not status_code:
            return dict(status=status)
        tmp_name = 'tmp_model'

        # copy the model file
        model_path = self.model_path()
        tmp_model_path = model_path.replace('model.py','%s.py' % tmp_name)
        try:
            shutil.copy(model_path, tmp_model_path)
        except IOError, e:
            return dict(status='Failed to create a temporary model file: %s' % e)

        model = self.get_model_name()
        package = __import__(get_package_name(), {}, {},
            [tmp_name])
        tmp_model = getattr(package, tmp_name)

        ok = []
        fail = []
        classes = order.split(',')
        for class_name in classes:
            try:
                obj = getattr(tmp_model, class_name)
                obj.createTable(ifNotExists=True)
                setattr(model, class_name, obj)
                ok.append('Table created for class %s' % class_name)
            except Exception, e:
                fail.append('Failed to create table for class %s: %s'
                    % (class_name,e))

        if len(ok):
            status += '\nTables Created: \n%s' % '\n'.join(ok)
        if len(fail):
            status += '\nFailed to create Tables:\n%s' % '\n'.join(fail)

        try:
            os.remove(tmp_model_path)
        except IOError, e:
            print "Fail to remove temporary model file: %s" % e

        return dict(status=status)

    def load_session_list(self):
        sessions_directory = pkg_resources.resource_filename(
            __name__, os.path.join('static', 'sessions'))
        return [x.replace('.js','')
            for x in os.listdir(sessions_directory) if x.endswith('.js')]

    def session_exists(self):
        return os.path.exists(session_file_name)

    [expose(format='json')]
    def retrieve_sample(self, name):
        sessions_directory = pkg_resources.resource_filename(
            __name__, os.path.join('static', 'sessions'))
        full_path = os.path.join(sessions_directory,'%s.js' % name)
        if not os.path.exists(full_path):
            return dict()
        return dict(session=open(full_path,'r').read())

    [expose(format='json')]
    def current_session_model(self):
        if not self.session_exists():
            return dict()
        return dict(session=open(session_file_name, 'r').read())

    [expose(format='json')]
    def save(self, code):
        status_code, status = self.save_model(code)
        return dict(status=status)

    [expose(format='json')]
    def session_list(self):
        return dict(models=self.load_session_list())

    [expose(template='%s.modelDesigner' % baseTemplate)]
    def index(self):
        self.register_static_directory()
        self.model = self.get_model_name()
        model_exists = None
        if self.model:
            model_exists = 1
        return dict(model_exists=model_exists,
            session_file_exists = self.session_exists(),
            session_list = self.load_session_list(),
            model_name=get_project_name())



#!/bin/env python
"""TurboGears installer"""

from distutils import log

# this is the TG version
TGVERSION = "1.0.10"
TGDOWNLOAD = "http://www.turbogears.org/%s/downloads/%s/" % (
    TGVERSION[:3], TGVERSION)

#!python
"""Bootstrap setuptools installation

If you want to use setuptools in your package's setup.py, just include this
file in the same directory with it, and add this to the top of your setup.py::

    from ez_setup import use_setuptools
    use_setuptools()

If you want to require a specific version of setuptools, set a download
mirror, or use an alternate download directory, you can do so by supplying
the appropriate options to ``use_setuptools()``.

This file can also be run as a script to install or upgrade setuptools.
"""
import sys
# this is the default version of setuptools, not TG
DEFAULT_VERSION = "0.6c11"
DEFAULT_URL     = "http://pypi.python.org/packages/%s/s/setuptools/" % sys.version[:3]

md5_data = {
    'setuptools-0.6b1-py2.3.egg': '8822caf901250d848b996b7f25c6e6ca',
    'setuptools-0.6b1-py2.4.egg': 'b79a8a403e4502fbb85ee3f1941735cb',
    'setuptools-0.6b2-py2.3.egg': '5657759d8a6d8fc44070a9d07272d99b',
    'setuptools-0.6b2-py2.4.egg': '4996a8d169d2be661fa32a6e52e4f82a',
    'setuptools-0.6b3-py2.3.egg': 'bb31c0fc7399a63579975cad9f5a0618',
    'setuptools-0.6b3-py2.4.egg': '38a8c6b3d6ecd22247f179f7da669fac',
    'setuptools-0.6b4-py2.3.egg': '62045a24ed4e1ebc77fe039aa4e6f7e5',
    'setuptools-0.6b4-py2.4.egg': '4cb2a185d228dacffb2d17f103b3b1c4',
    'setuptools-0.6c1-py2.3.egg': 'b3f2b5539d65cb7f74ad79127f1a908c',
    'setuptools-0.6c1-py2.4.egg': 'b45adeda0667d2d2ffe14009364f2a4b',
    'setuptools-0.6c10-py2.3.egg': 'ce1e2ab5d3a0256456d9fc13800a7090',
    'setuptools-0.6c10-py2.4.egg': '57d6d9d6e9b80772c59a53a8433a5dd4',
    'setuptools-0.6c10-py2.5.egg': 'de46ac8b1c97c895572e5e8596aeb8c7',
    'setuptools-0.6c10-py2.6.egg': '58ea40aef06da02ce641495523a0b7f5',
    'setuptools-0.6c11-py2.3.egg': '2baeac6e13d414a9d28e7ba5b5a596de',
    'setuptools-0.6c11-py2.4.egg': 'bd639f9b0eac4c42497034dec2ec0c2b',
    'setuptools-0.6c11-py2.5.egg': '64c94f3bf7a72a13ec83e0b24f2749b2',
    'setuptools-0.6c11-py2.6.egg': 'bfa92100bd772d5a213eedd356d64086',
    'setuptools-0.6c11-py2.7.egg': 'fe1f997bc722265116870bc7919059ea',
    'setuptools-0.6c2-py2.3.egg': 'f0064bf6aa2b7d0f3ba0b43f20817c27',
    'setuptools-0.6c2-py2.4.egg': '616192eec35f47e8ea16cd6a122b7277',
    'setuptools-0.6c3-py2.3.egg': 'f181fa125dfe85a259c9cd6f1d7b78fa',
    'setuptools-0.6c3-py2.4.egg': 'e0ed74682c998bfb73bf803a50e7b71e',
    'setuptools-0.6c3-py2.5.egg': 'abef16fdd61955514841c7c6bd98965e',
    'setuptools-0.6c4-py2.3.egg': 'b0b9131acab32022bfac7f44c5d7971f',
    'setuptools-0.6c4-py2.4.egg': '2a1f9656d4fbf3c97bf946c0a124e6e2',
    'setuptools-0.6c4-py2.5.egg': '8f5a052e32cdb9c72bcf4b5526f28afc',
    'setuptools-0.6c5-py2.3.egg': 'ee9fd80965da04f2f3e6b3576e9d8167',
    'setuptools-0.6c5-py2.4.egg': 'afe2adf1c01701ee841761f5bcd8aa64',
    'setuptools-0.6c5-py2.5.egg': 'a8d3f61494ccaa8714dfed37bccd3d5d',
    'setuptools-0.6c6-py2.3.egg': '35686b78116a668847237b69d549ec20',
    'setuptools-0.6c6-py2.4.egg': '3c56af57be3225019260a644430065ab',
    'setuptools-0.6c6-py2.5.egg': 'b2f8a7520709a5b34f80946de5f02f53',
    'setuptools-0.6c7-py2.3.egg': '209fdf9adc3a615e5115b725658e13e2',
    'setuptools-0.6c7-py2.4.egg': '5a8f954807d46a0fb67cf1f26c55a82e',
    'setuptools-0.6c7-py2.5.egg': '45d2ad28f9750e7434111fde831e8372',
    'setuptools-0.6c8-py2.3.egg': '50759d29b349db8cfd807ba8303f1902',
    'setuptools-0.6c8-py2.4.egg': 'cba38d74f7d483c06e9daa6070cce6de',
    'setuptools-0.6c8-py2.5.egg': '1721747ee329dc150590a58b3e1ac95b',
    'setuptools-0.6c9-py2.3.egg': 'a83c4020414807b496e4cfbe08507c03',
    'setuptools-0.6c9-py2.4.egg': '260a2be2e5388d66bdaee06abec6342a',
    'setuptools-0.6c9-py2.5.egg': 'fe67c3e5a17b12c0e7c541b7ea43a8e6',
    'setuptools-0.6c9-py2.6.egg': 'ca37b1ff16fa2ede6e19383e7b59245a',
}

import sys, os
try: from hashlib import md5
except ImportError: from md5 import md5

# TGBEGIN
win32 = """
The recommended Python for TurboGears 1.0 is Python 2.5.2.
You can download it here:

http://www.python.org/ftp/python/2.5.2/python-2.5.2.msi
"""

darwin = """
The recommended Python for TurboGears 1.0 is Python 2.5.2.
You can download a Universal build here:

http://www.python.org/ftp/python/2.5.2/python-2.5.2-macosx.dmg
"""

other = """
The recommended Python for TurboGears 1.0 is Python 2.5.2.
This version of Python is often available in your operating system's
native package format (via apt-get or yum, for instance).
You can also easily build Python from source on Unix-like systems.
Here is the source download link for Python:

http://www.python.org/ftp/python/2.5.2/Python-2.5.2.tar.bz2
"""

def get_python():
    """Displays information about where to download Python for your system.
    Currently, this will always return False. The idea is that eventually
    it may actually download Python if you want it to (in which case, it
    would return True)."""

    if sys.platform == "darwin":
        print darwin

    elif sys.platform == "win32":
        print win32

    else:
        print other

    return False

def get_yesno(default="y"):
    """Gets a valid 'y' or 'n' answer from the user."""
    while True:
        fromuser = raw_input("(y|n, default %s)" % default)
        if fromuser:
            val = fromuser[0].lower()
            if val == "y" or val == "n":
                break
        else:
            val = default
            break
    return val

def check_subprocess():
    # specific check for python 2.3 on Windows
    major, minor, minor2, tag, sub = sys.version_info
    if sys.platform == "win32" and major == 2 and minor == 3:
        try:
            import _subprocess

        except ImportError:
            print """Windows subprocess module must be installed, please download and install:
            http://effbot.org/downloads/subprocess-0.1-20041012.win32-py2.3.exe
            """
            # get out we cannot continue
            sys.exit(1)

def check_python():
    if sys.version_info < (2, 3):
        print "Python 2.3 or higher is required to run TurboGears 1.0."

        if not get_python():
            sys.exit(1)

    elif sys.version_info < (2, 4):
        print "TurboGears 1.0 supports Python 2.3, but 2.4 is strongly recommended."

        if not get_python():
            print "Would you like to continue with a Python 2.3 install?yes/no[n]"
            c = get_yesno(default="n")
            if c == "n":
                print "You can rerun the installer once you have an updated Python."
                sys.exit(1)

    elif sys.version_info >= (2, 6):
        print "Python 2.6 or higher is not supported by TurboGears 1.0."

        if not get_python():
            sys.exit(1)

def check_directory():
    """Ensures that the current directory does not contain a TurboGears
    directory that setuptools will attempt to install from."""

    if os.path.exists("turbogears") or os.path.exists("TurboGears"):
        print """There is a 'turbogears' directory in the current directory.
        EasyInstall will normally try to install from this directory rather
        than picking up the installation package from the net."""

        if os.access("..", os.W_OK) and not \
            os.path.exists(os.path.join("..", "turbogears")) and not \
            os.path.exists(os.path.join("..", "TurboGears")):
            print """
This installer needs a place to store the setuptools installation
package to do its work. The parent directory of this one is writeable.
Should the installation proceed from there ('n' to exit)?"""

            upone = get_yesno()
            if upone == "n":
                sys.exit(1)
            os.chdir("..")

        else:
            print """
This installer needs to run in a directory that does not contain a
'turbogears' directory and it also needs to run in a directory that
you have write access to. Please change directories and rerun this
installer.
"""
            sys.exit(1)

def tg_main():
    print "TurboGears Installer"
    check_python()
    check_subprocess()
    check_directory()
    print "Beginning setuptools/EasyInstall installation and TurboGears " \
          "download\n"
    if len(sys.argv) > 1:
        args = sys.argv[1:]

    else:
        args = []

    main(args)

    if post_install_msgs:
        print """
Important additional install information
----------------------------------------
"""
        print "\n\n".join(post_install_msgs)

post_install_msgs = []

def install_command():
    try:
        from setuptools.command import easy_install as eimodule

    except ImportError:
        import easy_install as eimodule

    original_class = eimodule.easy_install
    class easy_install(eimodule.easy_install):
        original_class.user_options.insert(0, ("future", None,
                                "use Genshi and SQLAlchemy"))
        original_class.boolean_options.append("future")
        def initialize_options(self):
            self.future = False
            original_class.initialize_options(self)

        def finalize_options(self):
            if self.future:
                self.args.append("TurboGears[future] == %s" % TGVERSION)
            else:
                self.args.append("TurboGears == %s" % TGVERSION)
            if self.upgrade is None:
                self.upgrade = True
            if self.script_dir is None:
                if os.name == "posix":
                    if self.prefix:
                        #honor the prefix specified on the command line
                        self.script_dir = os.path.join(self.prefix, "bin")
                else:
                    path = os.environ.get("PATH", "")
                    import re
                    if not re.search(r'Python\d+\\Script', path):
                        py_ver = "%i%i" % sys.version_info[:2]
                        post_install_msgs.append((
                          "You may need to update the PATH in your "
                          'environment to allow you to run the "tg-admin" '
                          "command or any other Python scripts that you "
                          "install.\n\nTo do that on Windows:\n\n"
                          "Go to System Properties -> Advanced -> Environment"
                          " Variables and add or edit the PATH variable to "
                          "include "
                          "';C:\Python%(py_ver)s;C:\Python%(py_ver)s\Scripts'"
                          "assuming you installed Python to the default "
                          "location.") % {'py_ver': py_ver})

            if self.find_links is None:
                self.find_links = []
            elif isinstance(self.find_links, basestring):
                self.find_links = [self.find_links]
            self.find_links.append(TGDOWNLOAD)
            original_class.finalize_options(self)

    # install our new class
    eimodule.easy_install = easy_install
# TGEND

def _validate_md5(egg_name, data):
    if egg_name in md5_data:
        digest = md5(data).hexdigest()
        if digest != md5_data[egg_name]:
            print >>sys.stderr, (
                "md5 validation of %s failed!  (Possible download problem?)"
                % egg_name
            )
            sys.exit(2)
    return data

def use_setuptools(version=DEFAULT_VERSION, download_base=DEFAULT_URL,
    to_dir=os.curdir, download_delay=15):
    """Automatically find/download setuptools and make it available on sys.path

    `version` should be a valid setuptools version number that is available
    as an egg for download under the `download_base` URL (which should end with
    a '/').  `to_dir` is the directory where setuptools will be downloaded, if
    it is not already available.  If `download_delay` is specified, it should
    be the number of seconds that will be paused before initiating a download,
    should one be required.  If an older version of setuptools is installed,
    this routine will print a message to ``sys.stderr`` and raise SystemExit in
    an attempt to abort the calling script.
    """
    was_imported = 'pkg_resources' in sys.modules or 'setuptools' in sys.modules
    def do_download():
        egg = download_setuptools(version, download_base, to_dir, download_delay)
        sys.path.insert(0, egg)
        import setuptools; setuptools.bootstrap_install_from = egg
    try:
        import pkg_resources
    except ImportError:
        return do_download()
    try:
        pkg_resources.require("setuptools>="+version); return
    except pkg_resources.VersionConflict, e:
        if was_imported:
            print >>sys.stderr, (
            "The required version of setuptools (>=%s) is not available, and\n"
            "can't be installed while this script is running. Please install\n"
            " a more recent version first, using 'easy_install -U setuptools'."
            "\n\n(Currently using %r)"
            ) % (version, e.args[0])
            sys.exit(2)
        else:
            del pkg_resources, sys.modules['pkg_resources']    # reload ok
            return do_download()
    except pkg_resources.DistributionNotFound:
        return do_download()

def download_setuptools(
    version=DEFAULT_VERSION, download_base=DEFAULT_URL, to_dir=os.curdir,
    delay = 15
):
    """Download setuptools from a specified location and return its filename

    `version` should be a valid setuptools version number that is available
    as an egg for download under the `download_base` URL (which should end
    with a '/'). `to_dir` is the directory where the egg will be downloaded.
    `delay` is the number of seconds to pause before an actual download attempt.
    """
    import urllib2, shutil
    egg_name = "setuptools-%s-py%s.egg" % (version,sys.version[:3])
    url = download_base + egg_name
    saveto = os.path.join(to_dir, egg_name)
    src = dst = None
    if not os.path.exists(saveto):  # Avoid repeated downloads
        try:
            from distutils import log
            if delay:
                log.warn("""
---------------------------------------------------------------------------
This script requires setuptools version %s to run (even to display
help).  I will attempt to download it for you (from
%s), but
you may need to enable firewall access for this script first.
I will start the download in %d seconds.

(Note: if this machine does not have network access, please obtain the file

   %s

and place it in this directory before rerunning this script.)
---------------------------------------------------------------------------""",
                    version, download_base, delay, url
                ); from time import sleep; sleep(delay)
            log.warn("Downloading %s", url)
            src = urllib2.urlopen(url)
            # Read/write all in one block, so we don't create a corrupt file
            # if the download is interrupted.
            data = _validate_md5(egg_name, src.read())
            dst = open(saveto,"wb"); dst.write(data)
        finally:
            if src: src.close()
            if dst: dst.close()
    return os.path.realpath(saveto)

def main(argv, version=DEFAULT_VERSION):
    """Install or upgrade setuptools and EasyInstall"""
    #TGBEGIN
    try:
        try:
            import setuptools
        except ImportError:
            egg = None
            try:
                egg = download_setuptools(version, delay=0)
                sys.path.insert(0,egg)
                from setuptools.command.easy_install import main
                return main(list(argv)+[egg])   # we're done here
            finally:
                if egg and os.path.exists(egg):
                    os.unlink(egg)
    except Exception, e:
        print >>sys.stderr, (
        "Could not download and install setupools:\n%s\n"
        "Please try installing setuptools manually."
        % (str(e) or e.__class__.__name__))
        sys.exit(2)
    if setuptools.__version__ == '0.0.1':
        print >>sys.stderr, (
        "You have an obsolete version of setuptools installed.  Please\n"
        "remove it from your system entirely before rerunning this script."
        )
        sys.exit(2)
    #TGEND

    req = "setuptools>=" + version
    import pkg_resources
    try:
        pkg_resources.require(req)
    except pkg_resources.VersionConflict:
        try:
            from setuptools.command.easy_install import main
        except ImportError:
            from easy_install import main
        #TGBEGIN
        install_command()
        #TGEND
        main(list(argv)+[download_setuptools(delay=0)])
        sys.exit(0) # try to force an exit
    else:
        from setuptools.command.easy_install import main
        #TGBEGIN
        install_command()
        #TGEND
        main(argv)

def update_md5(filenames):
    """Update our built-in md5 registry"""

    import re

    for name in filenames:
        base = os.path.basename(name)
        f = open(name,'rb')
        md5_data[base] = md5(f.read()).hexdigest()
        f.close()

    data = ["    %r: %r,\n" % it for it in md5_data.items()]
    data.sort()
    repl = "".join(data)

    import inspect
    srcfile = inspect.getsourcefile(sys.modules[__name__])
    f = open(srcfile, 'rb'); src = f.read(); f.close()

    match = re.search("\nmd5_data = {\n([^}]+)}", src)
    if not match:
        print >>sys.stderr, "Internal error!"
        sys.exit(2)

    src = src[:match.start(1)] + repl + src[match.end(1):]
    f = open(srcfile,'w')
    f.write(src)
    f.close()

if __name__=='__main__':
    if len(sys.argv) > 2 and sys.argv[1] == '--md5update':
        update_md5(sys.argv[2:])
    else:
        #TGBEGIN
        tg_main()
        #TGEND



import unittest

from dispatch import generic

from turbogears.genericfunctions import *


mo_state = []

[generic(MultiorderGenericFunction)]
def mo(a):
    pass

[mo.when("a>0", order=2)]
def mo2(next_method, a):
    mo_state.append(2)

[mo.when("a>0")]
def mo0(next_method, a):
    mo_state.append(0)
    return next_method(a)

[mo.when("a>0", order=1)]
def mo1(next_method, a):
    mo_state.append(1)
    return next_method(a)

[mo.around("a<23")]
def moa0(next_method, a):
    mo_state.append("a")
    return next_method(a)

[mo.around("a<123")]
def moa1(next_method, a):
    mo_state.append("b")
    return next_method(a)


class TestGenericFunctions(unittest.TestCase):

    def test_multiorder(self):
        mo(2)
        self.failUnless(mo_state == ["a", "b", 0,1,2])

    def test_getter(self):
        foo = 123
        _get_foo = getter("foo")
        _get_bar = getter("bar")
        self.failUnless(_get_foo() == 123)
        foo = 2
        self.failUnless(_get_foo() == 2)
        bar = 67
        self.failUnless(_get_bar() == 67)



import re
import cherrypy

import turbogears.validators as validators
import turbogears.widgets as widgets

from turbogears.testutil import catch_validation_errors
from turbogears.widgets.meta import copy_schema


oldrequest = None
int_validator = validators.Int(if_empty=None)
s_validator = validators.Schema(age=int_validator, ignore_key_missing=True)


class Request:
    validation_errors = {}

def setup_module():
    global oldrequest
    oldrequest = cherrypy.request
    cherrypy.request = Request()

def teardown_module():
    global oldrequest
    cherrypy.request = oldrequest


#XXX: We ignore missing keys to make passing value easier in tests
class TestSchema(validators.Schema):
    ignore_key_missing = True


class TestNestedWidgets:

    form = widgets.TableForm(name = "myform", fields=[
        widgets.TextField("name"),
        widgets.TextField("age", validator=int_validator),
        widgets.FieldSet("sub", fields = [
            widgets.TextField("name"),
            widgets.TextField("age", validator=int_validator),
            widgets.FieldSet("sub2", fields = [
                widgets.TextField("name"),
                widgets.TextField("age",
                    validator=int_validator),
            ], validator = TestSchema()),
        ], validator = TestSchema()),
    ], validator = TestSchema())

    def test_display(self):
        """
        Checks if names fo the widgets are set correctly depending on their
        path.
        """
        output = self.form.render(dict(sub=dict(sub2=dict(age=22))), format='xhtml')
        value_p = 'value="22"'
        name_p = 'name="sub.sub2.age"'
        assert (re.compile('.*'.join([value_p, name_p])).search(output) or
                re.compile('.*'.join([name_p, value_p])).search(output))

        output = self.form.render(dict(sub=dict(age=22)), format='xhtml')
        value_p = 'value="22"'
        name_p = 'name="sub.age"'
        assert (re.compile('.*'.join([value_p, name_p])).search(output) or
                re.compile('.*'.join([name_p, value_p])).search(output))

        output = self.form.render(dict(sub=dict(age=22)), format='xhtml')
        id_p = 'id="myform_sub_age"'
        name_p = 'name="sub.age"'
        assert (re.compile('.*'.join([value_p, id_p])).search(output) or
                re.compile('.*'.join([id_p, value_p])).search(output))

        output = self.form.render(dict(age=22), format='xhtml')
        value_p = 'value="22"'
        name_p = 'name="age"'
        assert (re.compile('.*'.join([value_p, name_p])).search(output) or
                re.compile('.*'.join([name_p, value_p])).search(output))

    def test_validate_outermost(self):
        values = dict(age="twenty")
        values, errors = catch_validation_errors(self.form, values)

        print values, errors

        assert errors.pop('age', False)
        assert not errors

    def test_validate_sub(self):
        values = dict(sub=dict(age="twenty"))

        values, errors = catch_validation_errors(self.form, values)

        print values, errors
        # check the outermost dict is not poluted with errors from the inner
        # dicts
        assert not errors.has_key('age')

        errors = errors['sub']
        assert errors.pop('age', False)
        assert not errors

    def test_validate_sub2(self):
        values = dict(sub=dict(sub2=dict(age="twenty")))

        values, errors = catch_validation_errors(self.form, values)

        print values, errors
        assert not errors.has_key('age')

        errors = errors['sub']
        print values, errors
        assert not errors.has_key('age')

        errors = errors['sub2']
        print values, errors
        assert errors.pop('age', False)
        assert not errors

    def test_validate_sub_and_sub2(self):
        values = dict(sub=dict(age="fhg", sub2=dict(age="twenty")))

        values, errors = catch_validation_errors(self.form, values)

        print values, errors
        assert not errors.has_key('age')

        errors = errors['sub']
        print values, errors
        assert errors.pop('age', False)

        errors = errors['sub2']
        print values, errors
        assert errors.pop('age', False)
        assert not errors

    def test_good_values(self):
        values = dict(age=22, sub=dict(sub2=dict(age=20)))

        values, errors = catch_validation_errors(self.form, values)

        print values, errors

        assert errors == {}
        assert values['age'] == 22

    def test_good_and_bad_values(self):
        values = dict(age="ddd", sub=dict(age="20", sub2=dict()))

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert errors.pop('age', False)
        assert not errors
        #assert values['sub']['age'] == 20


class TestNestedWidgetsWSchemaValidation:

    form = widgets.TableForm(
        name = "myform",
        validator = s_validator,
        fields=[
            widgets.TextField("name"),
            widgets.TextField("age"),
            widgets.FieldSet(
                name = "sub",
                validator = s_validator,
                fields = [
                    widgets.TextField("name"),
                    widgets.TextField("age"),
                    widgets.FieldSet(
                        name = "sub2",
                        validator = s_validator,
                        fields = [
                            widgets.TextField("name"),
                            widgets.TextField("age"),
                        ]
                    ),
                ]
            ),
        ]
    )

    def test_validate_sub_schema(self):
        values = dict(sub=dict(age="twenty"))

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        # check the outermost dict is not poluted with errors from the inner
        # dicts
        assert not errors.has_key('age')

        errors = errors['sub']
        assert errors.pop('age', False)
        assert not errors

    def test_good_and_bad_values_schema(self):
        values = dict(age="ddd", sub=dict(age="20", sub2=dict()))

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert errors.pop('age', False)
        assert not errors
        #assert values['sub']['age'] == 20

    def test_good_values_schema(self):
        values = dict(age=22, sub=dict(sub2=dict(age=20)))

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert errors == {}
        assert values['age'] == 22

    def test_validate_sub_and_sub2_schema(self):
        values = dict(sub=dict(age="fhg", sub2=dict(age="twenty")))

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert not errors.has_key('age')

        errors = errors['sub']
        print values, errors
        assert errors.pop('age', False)

        errors = errors['sub2']
        print values, errors
        assert errors.pop('age', False)
        assert not errors

    def test_validate_sub2_schema(self):
        values = dict(sub=dict(sub2=dict(age="twenty")))

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert not errors.has_key('age')

        errors = errors['sub']
        print values, errors
        assert not errors.has_key('age')

        errors = errors['sub2']
        print values, errors
        assert errors.pop('age', False)

    def test_validate_outermost_schema(self):
        values = dict(age="twenty")

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert errors.pop('age', False)
        assert not errors
        assert not errors


class TestNestedWidgetsWMixedValidation:

    form = widgets.TableForm(
        name = "myform",
        validator = s_validator,
        fields=[
            widgets.TextField("name"),
            widgets.TextField("age"),
            widgets.TextField("number", validator=int_validator),
            widgets.FieldSet(
                name = "sub",
                validator = s_validator,
                fields = [
                    widgets.TextField("name"),
                    widgets.TextField("age"),
                    widgets.TextField("number", validator=int_validator),
                    widgets.FieldSet(
                        name = "sub2",
                        fields = [
                            widgets.TextField("name"),
                            widgets.TextField("age", validator=int_validator),
                            widgets.TextField("number", validator=int_validator),
                        ]
                    ),
                ]
            ),
        ]
    )

    def test_mixed_validators(self):
        """
        Tests that schema validators and single validators can be mixed
        safely.
        """
        values = dict(
            age="bad",
            number="22",
            sub=dict(
                age="bad",
                number="bad",
                sub2=dict(
                    age="bad",
                    number="bad",
                )
            )
        )

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert errors.pop('age', False)
        #assert values['number'] == 22

        # assert errors are not getting poluted errors from other levels of
        # the tree
        assert errors.keys() == ['sub']
        errors = errors['sub']
        assert errors.pop('age', False)
        assert errors.pop('number', False)

        assert errors.keys() == ['sub2']
        errors = errors['sub2']
        assert errors.pop('age', False)
        assert errors.pop('number', False)
        assert not errors


class InnerSchema(validators.Schema):
    ignore_key_missing = True
    age = int_validator


class MiddleSchema(validators.Schema):
    ignore_key_missing = True
    age = int_validator
    sub2 = InnerSchema()


class OuterSchema(validators.Schema):
    ignore_key_missing = True
    age = int_validator
    sub = MiddleSchema()


class TestNestedSchemaValidators:

    #XXX: Age is always validated by the nested schemas, number is
    #     validated with widget validator.
    form = widgets.TableForm(
        name = "myform",
        validator = OuterSchema(),
        fields=[
            widgets.TextField("age"),
            widgets.TextField("number", validator=int_validator),
            widgets.FieldSet(
                name = "sub",
                fields = [
                    widgets.TextField("age"),
                    widgets.TextField("number", validator=int_validator),
                    widgets.FieldSet(
                        name = "sub2",
                        fields = [
                            widgets.TextField("age"),
                            widgets.TextField("number", validator=int_validator),
                        ]
                    ),
                ]
            ),
        ]
    )

    def test_nested_schemas(self):
        """
        Tests that we can nest schema validators safely.
        """
        values = dict(
            age="bad",
            number="22",
            sub=dict(
                age="27",
                number="bad",
                sub2=dict(
                    age="bad",
                    number="bad",
                )
            )
        )

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert errors.pop('age', False)
        #assert values['number'] == 22

        # assert errors are not getting poluted errors from other levels of
        # the tree
        assert errors.keys() == ['sub']
        errors = errors['sub']
        values = values['sub']
        #XXX This assertion fails :(
        #XXX But it's normal as the Schema doesn't convert good values in
        #    invalid Schemas, ATM
        #assert values['age'] == 27
        assert errors.pop('number', False)

        assert errors.keys() == ['sub2']
        errors = errors['sub2']
        assert errors.pop('age', False)
        assert errors.pop('number', False)
        assert not errors

    def test_nested_schemas_good_values(self):
        values = dict(
            age="21",
            number="22",
            sub=dict(
                age="27",
                number="28",
                sub2=dict(
                    age="33",
                    number="34",
                )
            )
        )

        values, errors = catch_validation_errors(self.form, values)
        print values, errors

        assert not errors
        assert (values["age"], values['number']) == (21, 22)

        values = values['sub']
        assert (values["age"], values['number']) == (27, 28)

        values = values['sub2']
        assert (values["age"], values['number']) == (33, 34)


def test_copy_schema():
    """Test that a validator schema can be copied."""
    class UserSchema(validators.Schema):
        user_name = validators.PlainText()
    schema = copy_schema(UserSchema())

def test_copy_nested_schema():
    """Test that a nested validator schema can be copied."""
    class PersonSchema(validators.Schema):
        class namefields(validators.Schema):
            firstname = validators.PlainText()
            lastname = validators.PlainText()
        class parents(validators.Schema):
            class father(validators.Schema):
                firstname = validators.PlainText()
                lastname = validators.PlainText()
            class mother(validators.Schema):
                firstname = validators.PlainText()
                lastname = validators.PlainText()
    schema = copy_schema(PersonSchema())



# Formatting configuration for locale de_CH

languages={'gv': 'Manx', 'gu': 'Gujarati', 'gd': u'Schottisch-G\xe4lisch', 'ga': 'Irisch', 'gn': 'Guarani', 'gl': 'Galizisch', 'la': 'Latein', 'ln': 'Lingala', 'lo': 'Laotisch', 'tt': 'Tatarisch', 'tr': u'T\xfcrkisch', 'ts': 'Tsonga', 'lv': 'Lettisch', 'to': 'Tongaisch', 'lt': 'Litauisch', 'tk': 'Turkmenisch', 'th': 'Thai', 'ti': 'Tigrinja', 'tg': 'Tadschikisch', 'te': 'Telugu', 'haw': 'Hawaiianisch', 'yi': 'Jiddisch', 'yo': 'Joruba', 'de': 'Deutsch', 'da': u'D\xe4nisch', 'dz': 'Bhutanisch', 'st': 'Sotho', 'qu': 'Quechua', 'el': 'Griechisch', 'eo': 'Esperanto', 'en': 'Englisch', 'zh': 'Chinesisch', 'za': 'Zhuang', 'uk': 'Ukrainisch', 'eu': 'Baskisch', 'et': 'Estnisch', 'es': 'Spanisch', 'ru': 'Russisch', 'rw': 'Ruandisch', 'rm': u'R\xe4toromanisch', 'rn': 'Kirundi', 'ro': u'Rum\xe4nisch', 'bn': 'Bengalisch', 'be': u'Wei\xdfrussisch', 'bg': 'Bulgarisch', 'ba': 'Baschkirisch', 'wo': 'Wolof', 'jv': 'Javanisch', 'bo': 'Tibetisch', 'bh': 'Biharisch', 'bi': 'Bislama', 'br': 'Bretonisch', 'ja': 'Japanisch', 'om': 'Oromo', 'oc': 'Okzitanisch', 'tw': 'Twi', 'or': 'Orija', 'xh': 'Xhosa', 'co': 'Korsisch', 'ca': 'Katalanisch', 'cy': 'Walisisch', 'cs': 'Tschechisch', 'zap': 'Zapotekisch', 'ps': 'Afghanisch (Paschtu)', 'pt': 'Portugiesisch', 'tl': 'Tagalog', 'pa': 'Pandschabisch', 'vi': 'Vietnamesisch', 'pl': 'Polnisch', 'hy': 'Armenisch', 'hr': 'Kroatisch', 'iu': 'Inukitut', 'hu': 'Ungarisch', 'hi': 'Hindi', 'ha': 'Hausa', 'he': u'Hebr\xe4isch', 'mg': 'Madagassisch', 'uz': 'Usbekisch', 'ml': 'Malayalam', 'mo': 'Moldauisch', 'mn': 'Mongolisch', 'mi': 'Maori', 'ik': 'Inupiak', 'mk': 'Mazedonisch', 'ur': 'Urdu', 'mt': 'Maltesisch', 'ms': 'Malaiisch', 'mr': 'Marathi', 'ug': 'Uigur', 'ta': 'Tamilisch', 'my': 'Burmesisch', 'aa': 'Afar', 'ab': 'Abchasisch', 'ss': 'Swazi', 'af': 'Afrikaans', 'tn': 'Setswana', 'sw': 'Suaheli', 'is': u'Isl\xe4ndisch', 'am': 'Amharisch', 'it': 'Italienisch', 'sv': 'Schwedisch', 'ia': 'Interlingua', 'as': 'Assamesisch', 'ar': 'Arabisch', 'su': 'Sudanesisch', 'zu': 'Zulu', 'ay': 'Aymara', 'az': 'Aserbaidschanisch', 'ie': 'Interlingue', 'id': 'Indonesisch', 'sk': 'Slowakisch', 'nl': u'Niederl\xe4ndisch', 'nn': 'Norwegisch Nynorsk', 'no': 'Norwegisch', 'na': 'Nauruisch', 'nb': u'Norwegisch Bokm\xe5l', 'ne': 'Nepalesisch', 'vo': u'Volap\xfck', 'fr': u'Franz\xf6sisch', 'sm': 'Samoanisch', 'fy': 'Friesisch', 'fa': 'Persisch', 'fi': 'Finnisch', 'fj': 'Fidschianisch', 'sa': 'Sanskrit', 'fo': u'F\xe4r\xf6isch', 'ka': 'Georgisch', 'kk': 'Kasachisch', 'sr': 'Serbisch', 'sq': 'Albanisch', 'ko': 'Koreanisch', 'kn': 'Kannada', 'km': 'Kambodschanisch', 'kl': u'Gr\xf6nl\xe4ndisch', 'ks': 'Kaschmirisch', 'si': 'Singhalesisch', 'sh': 'Serbo-Kroatisch', 'so': 'Somali', 'sn': 'Shona', 'ku': 'Kurdisch', 'sl': 'Slowenisch', 'ky': 'Kirgisisch', 'sg': 'Sango', 'sd': 'Sindhi'}

countries={'BD': 'Bangladesh', 'RW': 'Rwanda', 'DJ': 'Djibouti', 'BN': 'Brunei', 'MH': 'Marshall-Inseln', 'ST': u'Sao Tom\xe9 und Principe', 'BW': 'Botswana', 'GB': 'Grossbritannien', 'SB': 'Salomon-Inseln', 'ZW': 'Zimbabwe', 'CV': 'Kapverden'}

months=['Januar', 'Februar', u'M\xe4rz', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember']

abbrMonths=['Jan', 'Feb', 'Mrz', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez']

days=['Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag', 'Sonntag']

abbrDays=['Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa', 'So']

dateFormats={'medium': '%d.%m.%Y', 'full': '%%(dayname)s, %d. %%(monthname)s %Y', 'long': '%d. %%(monthname)s %Y', 'short': '%d.%m.%y'}

numericSymbols={'group': "'", 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



import os, glob, re

from cherrypy import config
from configobj import ConfigObj
import pkg_resources
import logging
import logging.handlers

__all__ = ["update_config", "get", "update"]

try:
    set
except NameError: # Python 2.3
    from sets import Set as set

class ConfigError(Exception):
    pass

def _get_formatters(formatters):
    for key, formatter in formatters.items():
        kw = {}
        fmt = formatter.get("format", None)
        if fmt:
            fmt = fmt.replace("*(", "%(")
            kw["fmt"] = fmt
        datefmt = formatter.get("datefmt", None)
        if datefmt:
            kw["datefmt"] = datefmt
        formatter = logging.Formatter(**kw)
        formatters[key] = formatter

def _get_handlers(handlers, formatters):
    for key, handler in handlers.items():
        kw = {}
        try:
            cls = handler.get("class")
            args = handler.get("args", tuple())
            level = handler.get("level", None)
            try:
                cls = eval(cls, logging.__dict__)
            except NameError:
                try:
                    cls = eval(cls, logging.handlers.__dict__)
                except NameError, err:
                    raise ConfigError("Specified class in handler "
                        "%s is not a recognizable logger name" % key)
            try:
                handler_obj = cls(*eval(args, logging.__dict__))
            except IOError,err:
                raise ConfigError("Missing or wrong argument to "
                    "%s in handler %s -> %s " % (cls.__name__,key,err))
            except TypeError,err:
                raise ConfigError("Wrong format for arguments "
                    "to %s in handler %s -> %s" % (cls.__name__,key,err))
            if level:
                level = eval(level, logging.__dict__)
                handler_obj.setLevel(level)
        except KeyError:
            raise ConfigError("No class specified for logging "
                "handler %s" % key)
        formatter = handler.get("formatter", None)
        if formatter:
            try:
                formatter = formatters[formatter]
            except KeyError:
                raise ConfigError("Handler %s references unknown "
                            "formatter %s" % (key, formatter))
            handler_obj.setFormatter(formatter)
        handlers[key] = handler_obj

def _get_loggers(loggers, handlers):
    for key, logger in loggers.items():
        qualname = logger.get("qualname", None)
        if qualname:
            log = logging.getLogger(qualname)
        else:
            log = logging.getLogger()

        level = logger.get("level", None)
        if level:
            level = eval(level, logging.__dict__)
        else:
            level = logging.NOTSET
        log.setLevel(level)

        propagate = logger.get("propagate", None)
        if propagate is not None:
            log.propagate = propagate

        cfghandlers = logger.get("handlers", None)
        if cfghandlers:
            if isinstance(cfghandlers, basestring):
                cfghandlers = [cfghandlers]
            for handler in cfghandlers:
                try:
                    handler = handlers[handler]
                except KeyError:
                    raise ConfigError("Logger %s references unknown "
                                "handler %s" % (key, handler))
                log.addHandler(handler)

def configure_loggers(config):
    """Configures the Python logging module, using options that are very
    similar to the ones listed in the Python documentation. This also
    removes the logging configuration from the configuration dictionary
    because CherryPy doesn't like it there. Here are some of the Python
    examples converted to the format used here:

    [logging]
    [[loggers]]
    [[[parser]]]
    [logger_parser]
    level="DEBUG"
    handlers="hand01"
    propagate=1
    qualname="compiler.parser"

    [[handlers]]
    [[[hand01]]]
    class="StreamHandler"
    level="NOTSET"
    formatter="form01"
    args="(sys.stdout,)"

    [[formatters]]
    [[[form01]]]
    format="F1 *(asctime)s *(levelname)s *(message)s"
    datefmt=


    One notable format difference is that *() is used in the formatter
    instead of %() because %() is already used for config file
    interpolation.
    """
    if not config.has_key("logging"):
        config["global"]["tg.new_style_logging"] = False
        return
    logcfg = config["logging"]
    formatters = logcfg.get("formatters", {})
    _get_formatters(formatters)

    handlers = logcfg.get("handlers", {})
    _get_handlers(handlers, formatters)

    loggers = logcfg.get("loggers", {})
    _get_loggers(loggers, handlers)

    del config["logging"]
    config["global"]["tg.new_style_logging"] = True

def config_defaults():
    current_dir_uri = os.path.abspath(os.getcwd())
    if not current_dir_uri.startswith('/'):
        current_dir_uri = '/' + current_dir_uri
    defaults = {'current_dir_uri' : current_dir_uri}
    return defaults

def config_obj(configfile = None, modulename = None):
    defaults = config_defaults()

    if modulename:
        firstdot = modulename.find('.')
        if firstdot < 0:
            raise ConfigError('Config file package not specified')
        lastdot = modulename.rfind('.')
        top_level_package = modulename[:firstdot]
        packagename = modulename[:lastdot]
        modname = modulename[lastdot+1:]
        modfile = pkg_resources.resource_filename(packagename,
                                        modname + '.cfg')
        if not os.path.exists(modfile):
            modfile = pkg_resources.resource_filename(packagename,
                                            modname)
        if os.path.isdir(modfile):
            configfiles = glob.glob(os.path.join(modfile, '*.cfg'))
        else:
            configfiles = [modfile]
        configdata = ConfigObj(unrepr=True)
        top_level_dir = os.path.normpath(pkg_resources.resource_filename(
            top_level_package, ''))
        package_dir = os.path.normpath(pkg_resources.resource_filename(
            packagename, ''))
        defaults.update(dict(top_level_dir=top_level_dir,
            package_dir=package_dir))
        configdata.merge(dict(DEFAULT=defaults))
        for file in configfiles:
            configdata2 = ConfigObj(file, unrepr=True)
            configdata2.merge(dict(DEFAULT=defaults))
            configdata.merge(configdata2)

    if configfile:
        if modulename:
            configdata2 = ConfigObj(configfile, unrepr=True)
            configdata2.merge(dict(DEFAULT=defaults))
            configdata.merge(configdata2)
        else:
            configdata = ConfigObj(configfile, unrepr=True)
    return configdata

def update_config(configfile=None, modulename=None):
    """Updates the system configuration either from a ConfigObj
    (INI-style) config file, a module name specified in dotted notation
    or both (the module name is assumed to have a ".cfg" extension).
    If both are specified, the module is called first,
    followed by the config file. This means that the config file's options
    override the options in the module file."""
    configdict = config_obj(configfile, modulename).dict()
    configure_loggers(configdict)
    config.update(configdict)

def get(key, default_value=None, return_section=False, path=None):
    """Retrieves a config value"""
    value = config.get(key, default_value, return_section, path)
    if value and key == 'sqlobject.dburi' and os.name == 'nt':
        value = re.sub('///(\w):', '///\\1|', value)
    return value

def update(configvalues):
    """Updates the configuration with the values from the dictionary."""
    return config.update(configvalues)



import Cookie
import cStringIO as StringIO
import logging
import os
import string
import types
import unittest

import cherrypy
from cherrypy import _cphttptools

try:
    import sqlobject
    from sqlobject.inheritance import InheritableSQLObject
except ImportError:
    sqlobject = None
try:
    import sqlalchemy
except ImportError:
    sqlalchemy = None

from turbogears import startup, config, update_config, \
    controllers, database, validators
from turbogears.identity import current_provider
from turbogears.util import get_model

# For clean tests, remove all compiled Kid templates
for w in os.walk('.'):
    if not os.sep + '.' in w[0]:
        for f in w[2]:
            if f.endswith('.kid'):
                f = os.path.join(w[0], f[:-3] + 'pyc')
                if os.path.exists(f):
                    os.remove(f)

# Load test configuration
if os.path.exists('test.cfg'):
    # Look for a 'config' package
    for dirpath, dirs, dummy2 in os.walk('.'):
        basename = os.path.basename(dirpath)
        dirname = os.path.basename(os.path.dirname(dirpath))
        init_py = os.path.join(dirpath, '__init__.py')
        if basename == 'config' and dirname[0] in string.ascii_letters + '_' \
                and os.path.exists(init_py):
            modulename = "%s.app" % dirpath[2:].replace(os.sep, ".")
            break
    else:
        modulename = None
    # XXX This is a temporary workaround, the code above to find the config
    # package should really be improved and moved elsewhere.
    # See http://trac.turbogears.org/ticket/2043
    try:
        update_config(configfile="test.cfg", modulename=modulename)
    except ImportError, exc:
        import warnings
        warnings.warn("Could not import configuration from module: %s" % exc,
            RuntimeWarning)
        update_config(configfile="test.cfg", modulename=None)
else:
    database.set_db_uri("sqlite:///:memory:")

config.update({'global':
        {'autoreload.on': False, 'tg.new_style_logging': True}})


def start_cp():
    if not config.get("cherrypy_started", False):
        cherrypy.server.start(serverClass=None, initOnly=True)
        config.update({"cherrypy_started" : True})


test_user = None

def set_identity_user(user):
    """Setup a user for configuring request's identity."""
    global test_user
    test_user = user


def attach_identity(req):
    if config.get("identity.on", False):
        req.identity = (test_user
            and current_provider.authenticated_identity(test_user)
            or current_provider.anonymous_identity())


def create_request(request, method="GET", protocol="HTTP/1.1",
        headers={}, rfile=None, clientAddress="127.0.0.1",
        remoteHost="localhost", scheme="http"):
    start_cp()
    if not rfile:
        rfile = StringIO.StringIO("")
    if type(headers) != dict:
        headerList = headers
    else:
        headerList = [(key, value) for key, value in headers.items()]
    headerList.append(("Host", "localhost"))
    if not hasattr(cherrypy.root, "started"):
        startup.startTurboGears()
        cherrypy.root.started = True
    req = _cphttptools.Request(clientAddress, 80, remoteHost, scheme)
    cherrypy.serving.request = req
    attach_identity(req)
    cherrypy.serving.response = _cphttptools.Response()
    req.run(" ".join((method, request, protocol)), headerList, rfile)

createRequest = create_request # deprecated


class BrowsingSession(object):

    def __init__(self):
        self.visit = None
        self.response, self.status = None, None
        self.cookie = Cookie.SimpleCookie()

    def goto(self, *args, **kwargs):
        if self.cookie:
            headers = kwargs.setdefault('headers', {})
            headers['Cookie'] = self.cookie.output()
        create_request(*args, **kwargs)
        # If we were given an encoding in the content type we should use it to
        # decode the response:
        ctype_parts = cherrypy.response.headers['Content-Type'].split(';')
        for parameter in ctype_parts[1:]:
            attribute, value = parameter.strip().split('=')
            try:
                self.unicode_response = cherrypy.response.body[0].decode(value)
                break
            except:
                # If the named encoding doesn't work then it doesn't work.  We
                # just won't create the unicode_response field.
                pass
        self.response = cherrypy.response.body[0]
        self.status = cherrypy.response.status
        if cherrypy.response.simple_cookie:
            self.cookie.update(cherrypy.response.simple_cookie)


def _return_directly(output, *args):
    return output


class DummySession:
    session_storage = dict
    to_be_loaded = None


class DummyRequest(_cphttptools.Request):
    """A very simple dummy request."""

    remote_host = "127.0.0.1"

    def __init__(self, method='GET', path='/', headers=None):
        super(DummyRequest, self).__init__("127.0.0.1", "4711", "127.0.0.1")
        self.headers = headers or {}
        self.method = method
        self.path = path
        self.path_info = ''
        self.query_string = ''
        self.base = ''
        self.params = {}
        self.object_trail = []
        self._session = DummySession()

    def purge__(self):
        pass


def call(method, *args, **kw):
    start_cp()
    output, response = call_with_request(method, DummyRequest(), *args, **kw)
    return output


def call_with_request(method, request, *args, **kw):
    """More fine-grained version of call method.

    This allows using request/response.

    """
    orig_proc_output = controllers._process_output
    controllers._process_output = _return_directly
    cherrypy.serving.response = _cphttptools.Response()
    cherrypy.serving.response.version = "1.1"
    cherrypy.serving.request = request
    if not hasattr(request, "identity"):
        attach_identity(request)
    output = None
    try:
        output = method(*args, **kw)
    finally:
        del cherrypy.serving.request
        controllers._process_output = orig_proc_output
    response = cherrypy.serving.response
    return output, response


class DBTest(unittest.TestCase):

    model = None

    def _get_soClasses(self):
        try:
            return [self.model.__dict__[x] for x in self.model.soClasses]
        except AttributeError:
            return self.model.__dict__.values()

    def setUp(self):
        if not self.model:
            self.model = get_model()
            if not self.model:
                raise "Unable to run database tests without a model"


        # list of constraints we will collect
        constraints = list()

        for item in self._get_soClasses():
            if isinstance(item, types.TypeType) and issubclass(item,
                sqlobject.SQLObject) and item != sqlobject.SQLObject \
                and item != InheritableSQLObject:
                # create table without applying constraints, collect
                # all the constaints for later creation.
                # see http://sqlobject.org/FAQ.html#mutually-referencing-tables
                # for more info
                collected_constraints = item.createTable(ifNotExists=True,
                        applyConstraints=False)

                if collected_constraints:
                    constraints.extend(collected_constraints)

        # now that all tables are created, add the constaints we collected
        for postponed_constraint in constraints:
            # item is the last processed item and we borrow its connection
            item._connection.query(postponed_constraint)


    def tearDown(self):
        database.rollback_all()
        for item in self._get_soClasses()[::-1]:
            if isinstance(item, types.TypeType) and issubclass(item,
                sqlobject.SQLObject) and item != sqlobject.SQLObject \
                and item != InheritableSQLObject:
                item.dropTable(ifExists=True)


def reset_cp():
    cherrypy.root = None


def catch_validation_errors(widget, value):
    """Catch and unpack validation errors (for testing purposes)."""
    try:
        value = widget.validate(value)
    except validators.Invalid, errors:
        try:
            errors = errors.unpack_errors()
        except AttributeError:
            pass
    else:
        errors = {}
    return value, errors


class MemoryListHandler(logging.Handler):

    def __init__(self):
        logging.Handler.__init__(self, level=logging.DEBUG)
        self.log = []

    def emit(self, record):
        print "Got record: %s" % record
        print "formatted as: %s" % self.format(record)
        self.log.append(self.format(record))

    def print_log(self):
        print "\n".join(self.log)
        self.log = []

    def get_log(self):
        log = self.log
        self.log = []
        return log

_memhandler = MemoryListHandler()


_currentcat = None


def capture_log(category):
    """Capture log for one category.

    The category can either be a single category (a string like 'foo.bar')
    or a list of them. You *must* call print_log() to reset when you're done.

    """
    global _currentcat
    assert not _currentcat
    if not isinstance(category, list) and not isinstance(category, tuple):
        category = [category]
    _currentcat = category
    for cat in category:
        log = logging.getLogger(cat)
        log.setLevel(logging.DEBUG)
        log.addHandler(_memhandler)


def _reset_logging():
    """Manage the resetting of the loggers."""
    global _currentcat
    if not _currentcat:
        return
    for cat in _currentcat:
        log = logging.getLogger(cat)
        log.removeHandler(_memhandler)
    _currentcat = None


def print_log():
    """Print the log captured by capture_log to stdout.

    Resets that log and resets the temporarily added handlers.

    """
    _reset_logging()
    _memhandler.print_log()


def get_log():
    """Return the list of log messages captured by capture_log.

    Resets that log and resets the temporarily added handlers.

    """
    _reset_logging()
    return _memhandler.get_log()


def sqlalchemy_cleanup():
    database.metadata.clear()
    try:
        database.metadata.dispose()
    except AttributeError: # not threadlocal
        if database.metadata.bind:
            database.metadata.bind.dispose()
    database._engine = None
    if database.mapper == database.session.mapper:
        # the following does not work for SA < 0.4
        sqlalchemy.orm.clear_mappers()


__all__ = ["call", "create_request", "createRequest", "DBTest",
    "attach_identity", "set_identity_user",
    "capture_log", "print_log", "get_log", "sqlalchemy_cleanup"]



# Formatting configuration for locale da_DK

languages={'el': u'Gr\xe6sk', 'gu': 'Gujaratisk', 'en': 'Engelsk', 'zh': 'Kinesisk', 'vi': 'Vietnamesisk', 'ca': 'Katalansk', 'am': 'Amharisk', 'it': 'Italiensk', 'cs': 'Tjekkisk', 'ar': 'Arabisk', 'ga': 'Irsk', 'eu': 'Baskisk', 'et': 'Estisk', 'gl': 'Galicisk', 'id': 'Indonesisk', 'es': 'Spansk', 'ru': 'Russisk', 'az': 'Aserbajdsjan', 'nl': 'Hollandsk', 'pt': 'Portugisisk', 'no': 'Norsk', 'tr': 'Tyrkisk', 'lv': 'Lettisk', 'lt': 'Litauisk', 'th': 'Thailandsk', 'so': 'Somalisk', 'he': 'Hebraisk', 'is': 'Islandsk', 'pl': 'Polsk', 'haw': 'Hawaii', 'be': 'Hviderussisk', 'fr': 'Fransk', 'bg': 'Bulgarsk', 'uk': 'Ukrainsk', 'sl': 'Slovensk', 'hr': 'Kroatisk', 'bn': 'Bengalsk', 'de': 'Tysk', 'da': 'Dansk', 'fa': 'Persisk', 'fi': 'Finsk', 'hy': 'Armensk', 'hu': 'Ungarsk', 'ja': 'Japansk', 'fo': u'F\xe6r\xf8sk', 'ka': 'Georgisk', 'ro': u'Rum\xe6nsk', 'kk': 'Kasakhisk', 'sr': 'Serbisk', 'sq': 'Albansk', 'ko': 'Koreansk', 'sv': 'Svensk', 'mk': 'Makedonsk', 'sk': 'Slovakisk', 'mt': 'Maltesisk', 'sh': 'Serbokroatisk', 'kw': 'Cornisk', 'ku': 'Kurdisk', 'mr': 'Marathisk', 'ta': 'Tamilsk'}

countries={'BD': 'Bangladesh', 'BE': 'Belgien', 'BF': 'Burkina Faso', 'BG': 'Bulgarien', 'BA': 'Bosnien-Hercegovina', 'BB': 'Barbados', 'WF': 'Wallis og Futuna', 'BM': 'Bermuda', 'BN': 'Brunei Darussalam', 'BO': 'Bolivia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BV': 'Bouvet Island', 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brasilien', 'BS': 'Bahamas', 'BY': 'Hviderusland', 'BZ': 'Belize', 'RU': 'Rusland', 'RW': 'Rwanda', 'TL': u'\xd8sttimor', 'RE': 'Reunion', 'TM': 'Turkmenistan', 'TJ': 'Tadsjikistan', 'RO': u'Rum\xe6nien', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': u'South Georgia og De Sydlige Sandwich\xf8er', 'GR': u'Gr\xe6kenland', 'GQ': u'\xc6kvatorialguinea', 'GP': 'Guadeloupe', 'JP': 'Japan', 'GY': 'Guyana', 'GF': 'Fransk Guyana', 'GE': 'Georgien', 'GD': 'Grenada', 'GB': 'Storbritannien', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': u'Gr\xf8nland', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunesien', 'JO': 'Jordan', 'SP': 'Serbia', 'HR': 'Kroatien', 'HT': 'Haiti', 'HU': 'Ungarn', 'HK': 'Hong Kong S.A.R., China', 'HN': 'Honduras', 'HM': 'Heard Island og McDonald Islands', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': u'De pal\xe6stinensiske omr\xe5der', 'PW': 'Palau', 'PT': 'Portugal', 'SJ': 'Svalbard og Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Irak', 'PA': 'Panama', 'PF': 'Fransk Polynesien', 'PG': 'Papua Ny Guinea', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Filippinerne', 'PN': 'Pitcairn', 'PL': 'Polen', 'PM': 'Saint Pierre og Miquelon', 'ZM': 'Zambia', 'EH': 'Vestsahara', 'EE': 'Estland', 'EG': 'Egypten', 'ZA': 'Sydafrika', 'EC': 'Ecuador', 'IT': 'Italien', 'VN': 'Vietnam', 'SB': u'Salomon\xf8erne', 'ET': 'Etiopien', 'SO': 'Somalia', 'ZW': 'Zimbabwe', 'SA': 'Saudi-Arabien', 'ES': 'Spanien', 'ER': 'Eritrea', 'MD': 'Republikken Moldova', 'MG': 'Madagaskar', 'MA': 'Marokko', 'MC': 'Monaco', 'UZ': 'Usbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Macao S.A.R. China', 'MN': 'Mongoliet', 'MH': u'Marshall\xf8erne', 'MK': 'Republikken Makedonien', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Maldiverne', 'MQ': 'Martinique', 'MP': 'Nordmarianerne', 'MS': 'Montserrat', 'MR': 'Mauretanien', 'UG': 'Uganda', 'MY': 'Malaysia', 'MX': 'Mexico', 'IL': 'Israel', 'FR': 'Frankrig', 'IO': 'Det Britiske Territorium i Det Indiske Ocean', 'SH': 'St. Helena', 'FI': 'Finland', 'FJ': 'Fiji', 'FK': u'Falklands\xf8erne', 'FM': 'Mikronesiens Forenede Stater', 'FO': u'F\xe6r\xf8erne', 'NI': 'Nicaragua', 'NL': 'Holland', 'NO': 'Norge', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Ny Caledonien', 'NE': 'Niger', 'NF': 'Norfolk Island', 'NG': 'Nigeria', 'NZ': 'New Zealand', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cook Islands', 'CI': 'Elfenbenskysten', 'CH': 'Schweiz', 'CO': 'Colombia', 'CN': 'Kina', 'CM': 'Cameroun', 'CL': 'Chile', 'CC': 'Cocos (Keeling) Islands', 'CA': 'Canada', 'CG': 'Congo', 'CF': 'Centralafrikanske Republik', 'CD': 'Den Demokratiske Republik Congo', 'CZ': 'Tjekkiet', 'CY': 'Cypern', 'CX': u'Jule\xf8en', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Kap Verde', 'CU': 'Cuba', 'SZ': 'Swaziland', 'SY': 'Syrien', 'KG': 'Kirgisistan', 'KE': 'Kenya', 'SR': 'Surinam', 'KI': 'Kiribati', 'KH': 'Cambodja', 'KN': 'Saint Kitts og Nevis', 'KM': 'Comorerne', 'ST': 'Sao Tome og Principe', 'SK': 'Slovakiet', 'KR': 'Sydkorea', 'SI': 'Slovenien', 'KP': 'Nordkorea', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychellerne', 'KZ': 'Kasakhstan', 'KY': u'Cayman\xf8erne', 'SG': 'Singapore', 'SE': 'Sverige', 'SD': 'Sudan', 'DO': 'Den Dominikanske Republik', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Danmark', 'VG': u'De britiske jomfru\xf8er', 'DE': 'Tyskland', 'YE': 'Yemen', 'DZ': 'Algeriet', 'US': 'Amerikas Forenede Stater', 'UY': 'Uruguay', 'YU': 'Jugoslavien', 'YT': 'Mayotte', 'UM': u'De Mindre Amerikanske Overs\xf8iske \xd8er', 'LB': 'Libanon', 'LC': 'Saint Lucia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad og Tobago', 'TR': 'Tyrkiet', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Letland', 'TO': 'Tonga', 'LT': 'Litauen', 'LU': 'Luxembourg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thailand', 'TF': 'Franske Besiddelser i Det Sydlige Indiske Ocean', 'TG': 'Togo', 'TD': 'Tchad', 'TC': 'Turks and Caicos Islands', 'LY': 'Libyen', 'VA': 'Vatikanstaten', 'VC': 'St. Vincent og Grenadinerne', 'AE': 'Forenede Arabiske Emirater', 'AD': 'Andorra', 'AG': 'Antigua og Barbuda', 'AF': 'Afghanistan', 'AI': 'Anguilla', 'VI': u'De amerikanske jomfru\xf8er', 'IS': 'Island', 'IR': 'Iran', 'AM': 'Armenien', 'AL': 'Albanien', 'AO': 'Angola', 'AN': 'Hollandske Antiller', 'AQ': 'Antarktis', 'AS': 'Amerikansk Samoa', 'AR': 'Argentina', 'AU': 'Australien', 'AT': u'\xd8strig', 'AW': 'Aruba', 'IN': 'Indien', 'TZ': 'Tanzania', 'AZ': 'Aserbajdsjan', 'IE': 'Irland', 'ID': 'Indonesien', 'UA': 'Ukraine', 'QA': 'Qatar', 'MZ': 'Mozambique'}

months=['januar', 'februar', 'marts', 'april', 'maj', 'juni', 'juli', 'august', 'september', 'oktober', 'november', 'december']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'maj', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'dec']

days=['mandag', 'tirsdag', 'onsdag', 'torsdag', 'fredag', u'l\xf8rdag', u's\xf8ndag']

abbrDays=['man', 'tir', 'ons', 'tor', 'fre', u'l\xf8r', u's\xf8n']

dateFormats={'medium': '%d-%m-%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d. %%(abbrmonthname)s %Y', 'short': '%d-%m-%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



# Formatting configuration for locale de_AT

languages={'gv': 'Manx', 'gu': 'Gujarati', 'gd': u'Schottisch-G\xe4lisch', 'ga': 'Irisch', 'gn': 'Guarani', 'gl': 'Galizisch', 'la': 'Latein', 'ln': 'Lingala', 'lo': 'Laotisch', 'tt': 'Tatarisch', 'tr': u'T\xfcrkisch', 'ts': 'Tsonga', 'lv': 'Lettisch', 'to': 'Tongaisch', 'lt': 'Litauisch', 'tk': 'Turkmenisch', 'th': 'Thai', 'ti': 'Tigrinja', 'tg': 'Tadschikisch', 'te': 'Telugu', 'haw': 'Hawaiianisch', 'yi': 'Jiddisch', 'yo': 'Joruba', 'de': 'Deutsch', 'da': u'D\xe4nisch', 'dz': 'Bhutanisch', 'st': 'Sotho', 'qu': 'Quechua', 'el': 'Griechisch', 'eo': 'Esperanto', 'en': 'Englisch', 'zh': 'Chinesisch', 'za': 'Zhuang', 'uk': 'Ukrainisch', 'eu': 'Baskisch', 'et': 'Estnisch', 'es': 'Spanisch', 'ru': 'Russisch', 'rw': 'Ruandisch', 'rm': u'R\xe4toromanisch', 'rn': 'Kirundi', 'ro': u'Rum\xe4nisch', 'bn': 'Bengalisch', 'be': u'Wei\xdfrussisch', 'bg': 'Bulgarisch', 'ba': 'Baschkirisch', 'wo': 'Wolof', 'jv': 'Javanisch', 'bo': 'Tibetisch', 'bh': 'Biharisch', 'bi': 'Bislama', 'br': 'Bretonisch', 'ja': 'Japanisch', 'om': 'Oromo', 'oc': 'Okzitanisch', 'tw': 'Twi', 'or': 'Orija', 'xh': 'Xhosa', 'co': 'Korsisch', 'ca': 'Katalanisch', 'cy': 'Walisisch', 'cs': 'Tschechisch', 'zap': 'Zapotekisch', 'ps': 'Afghanisch (Paschtu)', 'pt': 'Portugiesisch', 'tl': 'Tagalog', 'pa': 'Pandschabisch', 'vi': 'Vietnamesisch', 'pl': 'Polnisch', 'hy': 'Armenisch', 'hr': 'Kroatisch', 'iu': 'Inukitut', 'hu': 'Ungarisch', 'hi': 'Hindi', 'ha': 'Hausa', 'he': u'Hebr\xe4isch', 'mg': 'Madagassisch', 'uz': 'Usbekisch', 'ml': 'Malayalam', 'mo': 'Moldauisch', 'mn': 'Mongolisch', 'mi': 'Maori', 'ik': 'Inupiak', 'mk': 'Mazedonisch', 'ur': 'Urdu', 'mt': 'Maltesisch', 'ms': 'Malaiisch', 'mr': 'Marathi', 'ug': 'Uigur', 'ta': 'Tamilisch', 'my': 'Burmesisch', 'aa': 'Afar', 'ab': 'Abchasisch', 'ss': 'Swazi', 'af': 'Afrikaans', 'tn': 'Setswana', 'sw': 'Suaheli', 'is': u'Isl\xe4ndisch', 'am': 'Amharisch', 'it': 'Italienisch', 'sv': 'Schwedisch', 'ia': 'Interlingua', 'as': 'Assamesisch', 'ar': 'Arabisch', 'su': 'Sudanesisch', 'zu': 'Zulu', 'ay': 'Aymara', 'az': 'Aserbaidschanisch', 'ie': 'Interlingue', 'id': 'Indonesisch', 'sk': 'Slowakisch', 'nl': u'Niederl\xe4ndisch', 'nn': 'Norwegisch Nynorsk', 'no': 'Norwegisch', 'na': 'Nauruisch', 'nb': u'Norwegisch Bokm\xe5l', 'ne': 'Nepalesisch', 'vo': u'Volap\xfck', 'fr': u'Franz\xf6sisch', 'sm': 'Samoanisch', 'fy': 'Friesisch', 'fa': 'Persisch', 'fi': 'Finnisch', 'fj': 'Fidschianisch', 'sa': 'Sanskrit', 'fo': u'F\xe4r\xf6isch', 'ka': 'Georgisch', 'kk': 'Kasachisch', 'sr': 'Serbisch', 'sq': 'Albanisch', 'ko': 'Koreanisch', 'kn': 'Kannada', 'km': 'Kambodschanisch', 'kl': u'Gr\xf6nl\xe4ndisch', 'ks': 'Kaschmirisch', 'si': 'Singhalesisch', 'sh': 'Serbo-Kroatisch', 'so': 'Somali', 'sn': 'Shona', 'ku': 'Kurdisch', 'sl': 'Slowenisch', 'ky': 'Kirgisisch', 'sg': 'Sango', 'sd': 'Sindhi'}

countries={'BD': 'Bangladesch', 'BE': 'Belgien', 'BF': 'Burkina Faso', 'BG': 'Bulgarien', 'BA': 'Bosnien und Herzegowina', 'BB': 'Barbados', 'WF': 'Wallis und Futuna', 'BM': 'Bermuda', 'BN': 'Brunei Darussalam', 'BO': 'Bolivien', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaika', 'BV': 'Bouvetinsel', 'BW': 'Botsuana', 'WS': 'Samoa', 'BR': 'Brasilien', 'BS': 'Bahamas', 'BY': 'Belarus', 'BZ': 'Belize', 'RU': 'Russland', 'RW': 'Ruanda', 'TL': 'Osttimor', 'RE': u'R\xe9union', 'TM': 'Turkmenistan', 'TJ': 'Tadschikistan', 'RO': u'Rum\xe4nien', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': u'S\xfcdgeorgien und die S\xfcdlichen Sandwichinseln', 'GR': 'Griechenland', 'GQ': u'\xc4quatorialguinea', 'GP': 'Guadeloupe', 'JP': 'Japan', 'GY': 'Guyana', 'GF': u'Franz\xf6sisch-Guayana', 'GE': 'Georgien', 'GD': 'Grenada', 'GB': u'Vereinigtes K\xf6nigreich', 'GA': 'Gabun', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': u'Gr\xf6nland', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunesien', 'JO': 'Jordanien', 'SP': 'Serbien', 'HR': 'Kroatien', 'HT': 'Haiti', 'HU': 'Ungarn', 'HK': 'Hong Kong S.A.R., China', 'HN': 'Honduras', 'HM': 'Heard und McDonaldinseln', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': u'Pal\xe4stina', 'PW': 'Palau', 'PT': 'Portugal', 'SJ': 'Svalbard und Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Irak', 'PA': 'Panama', 'PF': u'Franz\xf6sisch-Polynesien', 'PG': 'Papua-Neuguinea', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Philippinen', 'PN': 'Pitcairn', 'PL': 'Polen', 'PM': 'St. Pierre und Miquelon', 'ZM': 'Sambia', 'EH': 'Westsahara', 'EE': 'Estland', 'EG': u'\xc4gypten', 'ZA': u'S\xfcdafrika', 'EC': 'Ecuador', 'IT': 'Italien', 'VN': 'Vietnam', 'SB': 'Salomonen', 'ET': u'\xc4thiopien', 'SO': 'Somalia', 'ZW': 'Simbabwe', 'SA': 'Saudi-Arabien', 'ES': 'Spanien', 'ER': 'Eritrea', 'MD': 'Moldawien', 'MG': 'Madagaskar', 'MA': 'Marokko', 'MC': 'Monaco', 'UZ': 'Usbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Macau S.A.R.,China', 'MN': 'Mongolei', 'MH': 'Marshallinseln', 'MK': 'Mazedonien', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Malediven', 'MQ': 'Martinique', 'MP': u'N\xf6rdliche Marianen', 'MS': 'Montserrat', 'MR': 'Mauretanien', 'UG': 'Uganda', 'MY': 'Malaysia', 'MX': 'Mexiko', 'IL': 'Israel', 'FR': 'Frankreich', 'IO': 'Britisches Territorium im Indischen Ozean', 'SH': 'St. Helena', 'FI': 'Finnland', 'FJ': 'Fidschi', 'FK': 'Falklandinseln', 'FM': 'Mikronesien', 'FO': u'F\xe4r\xf6er', 'NI': 'Nicaragua', 'NL': 'Niederlande', 'NO': 'Norwegen', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Neukaledonien', 'NE': 'Niger', 'NF': 'Norfolkinsel', 'NG': 'Nigeria', 'NZ': 'Neuseeland', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cookinseln', 'CI': u"C\xf4te d'Ivoire", 'CH': 'Schweiz', 'CO': 'Kolumbien', 'CN': 'China', 'CM': 'Kamerun', 'CL': 'Chile', 'CC': 'Kokosinseln (Keeling)', 'CA': 'Kanada', 'CG': 'Kongo', 'CF': 'Zentralafrikanische Republik', 'CD': 'Demokratische Republik Kongo', 'CZ': 'Tschechien', 'CY': 'Zypern', 'CX': 'Weihnachtsinsel', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Kap Verde', 'CU': 'Kuba', 'SZ': 'Swasiland', 'SY': 'Syrien', 'KG': 'Kirgisistan', 'KE': 'Kenia', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': 'Kambodscha', 'KN': 'St. Kitts und Nevis', 'KM': 'Komoren', 'ST': u'S\xe3o Tom\xe9 und Pr\xedncipe', 'SK': 'Slowakei', 'KR': 'Republik Korea', 'SI': 'Slowenien', 'KP': 'Demokratische Volksrepublik Korea', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychellen', 'KZ': 'Kasachstan', 'KY': 'Kaimaninseln', 'SG': 'Singapur', 'SE': 'Schweden', 'SD': 'Sudan', 'DO': 'Dominikanische Republik', 'DM': 'Dominica', 'DJ': 'Dschibuti', 'DK': u'D\xe4nemark', 'VG': 'Britische Jungferninseln', 'DE': 'Deutschland', 'YE': 'Jemen', 'DZ': 'Algerien', 'US': 'Vereinigte Staaten', 'UY': 'Uruguay', 'YU': 'Jugoslawien', 'YT': 'Mayotte', 'UM': 'Amerikanisch-Ozeanien', 'LB': 'Libanon', 'LC': 'St. Lucia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad und Tobago', 'TR': u'T\xfcrkei', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Lettland', 'TO': 'Tonga', 'LT': 'Litauen', 'LU': 'Luxemburg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thailand', 'TF': u'Franz\xf6sische S\xfcd- und Antarktisgebiete', 'TG': 'Togo', 'TD': 'Tschad', 'TC': 'Turks- und Caicosinseln', 'LY': 'Libyen', 'VA': 'Vatikanstadt', 'VC': 'St. Vincent und die Grenadinen', 'AE': 'Vereinigte Arabische Emirate', 'AD': 'Andorra', 'AG': 'Antigua und Barbuda', 'AF': 'Afghanistan', 'AI': 'Anguilla', 'VI': 'Amerikanische Jungferninseln', 'IS': 'Island', 'IR': 'Iran', 'AM': 'Armenien', 'AL': 'Albanien', 'AO': 'Angola', 'AN': u'Niederl\xe4ndische Antillen', 'AQ': 'Antarktis', 'AS': 'Amerikanisch-Samoa', 'AR': 'Argentinien', 'AU': 'Australien', 'AT': u'\xd6sterreich', 'AW': 'Aruba', 'IN': 'Indien', 'TZ': 'Tansania', 'AZ': 'Aserbaidschan', 'IE': 'Irland', 'ID': 'Indonesien', 'UA': 'Ukraine', 'QA': 'Katar', 'MZ': 'Mosambik'}

months=[u'J\xe4nner', 'Februar', u'M\xe4rz', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember']

abbrMonths=[u'J\xe4n', 'Feb', u'M\xe4r', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez']

days=['Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag', 'Sonntag']

abbrDays=['Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa', 'So']

dateFormats={'medium': '%d.%m.%Y', 'full': '%%(dayname)s, %d. %%(monthname)s %Y', 'long': '%d. %%(monthname)s %Y', 'short': '%d.%m.%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



"""Tests for paginate"""

import unittest
from urllib import quote
import warnings

from turbogears import config, expose, database
from turbogears.controllers import RootController, url
from turbogears.database import get_engine, metadata, session, mapper
from turbogears.paginate import paginate, sort_ordering, sort_data
from turbogears.testutil import create_request, sqlalchemy_cleanup
from turbojson.jsonify import jsonify

import cherrypy
from sqlalchemy import Table, Column, ForeignKey, Integer, String
from sqlalchemy.orm import relation
try:
    from sqlalchemy.ext.selectresults import SelectResults as SASelectResults
except ImportError: # SQLAlchemy >= 0.5
    SASelectResults = None
from sqlobject import SQLObject, IntCol, StringCol, connectionForURI

query_methods = 'Q QA SR SO SL'.split()
if SASelectResults is None:
    query_methods.remove('SR')

__connection__ = hub = database.PackageHub('test_paginate')


class listlike(object):
    """A minimum list-like object that is needed for paginate."""

    def __init__(self, *args):
        self._list = list(*args)

    def __len__(self):
        return len(self._list)

    def __getitem__(self, key):
        return self._list[key]


def test_sort_ordering():
    """Test the sort_ordering auxiliary function."""
    s = []
    sort_ordering(s, 'a')
    assert s == ['a']
    sort_ordering(s, 'a') # descending order
    assert s == ['-a']
    sort_ordering(s, 'b')
    assert s == ['b', '-a']
    sort_ordering(s, 'c')
    assert s == ['c', 'b', '-a']
    sort_ordering(s, 'c') # descending order
    assert s == ['-c', 'b', '-a']
    sort_ordering(s, 'a') # 'a' again... still in descending order
    assert s == ['-a', '-c', 'b']
    sort_ordering(s, 'a') # 'a' again... now in ascending order
    assert s == ['a', '-c', 'b']
    sort_ordering(s, 'd')
    assert s == ['d', 'a', '-c', 'b']
    sort_ordering(s, 'a')
    assert s == ['a', 'd', '-c', 'b']


def test_sort_in_memory():
    """Test sorting in memory only."""
    for access in range(2):
        if access:
            # ordinary attribute access with hashable keys
            class row(object):
                def __init__(self, ab):
                    self.ab = ab
                    self.a, self.b = self.ab = ab
                def __repr__(self):
                    return self.ab
                def __cmp__(self, other):
                    return cmp(self.ab, other.ab)
        else:
            # dictionary access with unhashable keys
            class row(dict):
                def __init__(self, ab):
                    self.ab = ab
                    self['a'], self['b'] = map(list, ab)
                def __repr__(self):
                    return self.ab
                def __cmp__(self, other):
                    return cmp(self.ab, other.ab)
        def make_data(rows):
            return map(row, rows.split())
        ab = make_data('aa ab ba bb')
        aB = make_data('ab aa bb ba')
        Ab = make_data('ba bb aa ab')
        AB = make_data('bb ba ab aa')
        ba = make_data('aa ba ab bb')
        bA = make_data('ba aa bb ab')
        Ba = make_data('ab bb aa ba')
        BA = make_data('bb ab ba aa')
        if access:
            assert not isinstance(ab[0], dict) \
                and getattr(ab[0], 'a', 'b') == 'a'
        else:
            assert isinstance(ab[0], dict) and ab[0].get('a', 'b') == ['a'] \
                and getattr(ab[0], 'a', 'b') == 'b'
        assert sort_data(ab, []) == ab
        assert sort_data(ab, ['b'], False) == ab
        assert sort_data(ab, ['a']) == ab
        assert sort_data(ab, ['a', 'b']) == ab
        assert sort_data(ab, ['a', '-b']) == aB
        assert sort_data(ab, ['b']) == ba
        assert sort_data(ab, ['b', 'a']) == ba
        assert sort_data(ab, ['b', '-a']) == bA
        assert sort_data(ab, ['-a']) == Ab
        assert sort_data(ab, ['-a', 'b']) == Ab
        assert sort_data(ab, ['-a', '-b']) == AB
        assert sort_data(ab, ['-b']) == Ba
        assert sort_data(ab, ['-b', 'a']) == Ba
        assert sort_data(ab, ['-b', '-a']) == BA
        assert sort_data(aB, ['a', 'b']) == ab
        assert sort_data(Ab, ['a', 'b']) == ab
        assert sort_data(AB, ['a', 'b']) == ab
        assert sort_data(ba, ['a', 'b']) == ab
        assert sort_data(bA, ['a', 'b']) == ab
        assert sort_data(Ba, ['a', 'b']) == ab
        assert sort_data(BA, ['a', 'b']) == ab
        assert sort_data(aB, ['b', 'a']) == ba
        assert sort_data(Ab, ['b', 'a']) == ba
        assert sort_data(AB, ['b', 'a']) == ba
        assert sort_data(ba, ['b', 'a']) == ba
        assert sort_data(bA, ['b', 'a']) == ba
        assert sort_data(Ba, ['b', 'a']) == ba
        assert sort_data(BA, ['b', 'a']) == ba
        assert sort_data(aB, ['-a', 'b']) == Ab
        assert sort_data(Ab, ['-a', 'b']) == Ab
        assert sort_data(AB, ['-a', 'b']) == Ab
        assert sort_data(ba, ['-a', 'b']) == Ab
        assert sort_data(bA, ['-a', 'b']) == Ab
        assert sort_data(Ba, ['-a', 'b']) == Ab
        assert sort_data(BA, ['-a', 'b']) == Ab
        assert sort_data(aB, ['b', '-a']) == bA
        assert sort_data(Ab, ['b', '-a']) == bA
        assert sort_data(AB, ['b', '-a']) == bA
        assert sort_data(ba, ['b', '-a']) == bA
        assert sort_data(bA, ['b', '-a']) == bA
        assert sort_data(Ba, ['b', '-a']) == bA
        assert sort_data(BA, ['b', '-a']) == bA


class Spy(object):
    """Helper class to test paginate's instances in cherrypy.request.

    We could use a special template to output paginate's attributes, but
    testing values/types before they are rendered (when possible) is much
    easier.
    """

    def __init__(self, name=None, **expectations):
        self.name = name
        self.expectations = expectations

    def __str__(self):
        if self.name:
            paginate = cherrypy.request.paginates[self.name]
        else:
            paginate = cherrypy.request.paginate
            assert paginate in cherrypy.request.paginates.values()
        for k, v in self.expectations.iteritems():
            if not hasattr(paginate, k):
                return "fail: paginate does not have '%s' attribute" % k
            w = getattr(paginate, k)
            if isinstance(v, xrange):
                v = list(v)
            if isinstance(w, xrange):
                w = list(w)
            if w != v:
                return "fail: expected %s=%r, got %s=%r" % (k, v, k, w)
        return "ok: [paginate %s ]" % \
            " , ".join(["%s=%r" % (k, v) for k, v in paginate.__dict__.items()])

    def assert_ok(body, key, value, raw=False):
        assert "ok: [paginate" in body
        if raw:
            expr = "%s=%s " % (key, value)
        else:
            expr = "%s=%r " % (key, value)
        # some simplejson versions escape forward slashes, so fix this
        body = body.replace('\\/', '/')
        if expr not in body:
            print body
        assert expr in body, "expected %s" % expr
    assert_ok = staticmethod(assert_ok)

[jsonify.when('isinstance(obj, Spy)')]
def jsonify_spy(obj):
    result = str(obj)
    return result


class TestSpy(unittest.TestCase):
    """Never trust a spy"""

    class MyRoot(RootController):

        [expose()]
        [paginate("data")]
        def spy(self):
            data = range(100)
            spy = Spy()
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data")]
        def spy_correct_expectation(self):
            data = range(100)
            spy = Spy(page_count=10)
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data")]
        def spy_wrong_expectation(self):
            data = range(100)
            spy = Spy(page_count=9)
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data")]
        def spy_invalid_expectation(self):
            data = range(100)
            spy = Spy(foobar=10)
            return dict(data=data, spy=spy)


    def setUp(self):
        cherrypy.root = self.MyRoot()

    def test_spy(self):
        create_request('/spy')
        body = cherrypy.response.body[0]
        Spy.assert_ok(body, 'current_page', 1)
        try:
            Spy.assert_ok(body, 'current_page', 2)
            raise Exception("above test should have failed")
        except AssertionError:
            pass

    def test_correct_expectation(self):
        create_request('/spy_correct_expectation')
        body = cherrypy.response.body[0]
        assert "ok: [paginate" in body

    def test_wrong_expectation(self):
        create_request('/spy_wrong_expectation')
        body = cherrypy.response.body[0]
        assert "fail: expected page_count=9, got page_count=10" in body

    def test_invalid_expectation(self):
        create_request('/spy_invalid_expectation')
        body = cherrypy.response.body[0]
        assert "fail: paginate does not have 'foobar' attribute" in body

    def test_raw_expectation(self):
        create_request('/spy_correct_expectation')
        Spy.assert_ok(cherrypy.response.body[0], 'var_name', 'data')
        Spy.assert_ok(cherrypy.response.body[0], 'var_name', "'data'", raw=True)


class TestPagination(unittest.TestCase):
    """Base class for all Paginate TestCases"""

    def setUp(self):
        cherrypy.root = self.MyRoot()

    def request(self, url):
        create_request(url)
        self.body = cherrypy.response.body[0]
        self.status = cherrypy.response.status
        if "fail: " in self.body:
            print self.body
            assert False, "Spy alert! Check body output for details..."


class TestBasicPagination(TestPagination):

    class MyRoot(RootController):

        [expose()]
        [paginate("data", limit=4)]
        def basic(self):
            spy = Spy(var_name='data', pages=xrange(1, 4), limit=4, page_count=3,
                      order=None, ordering=[], row_count=10)
            data = range(10)
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data", limit=4)]
        def listlike(self):
            spy = Spy(var_name='data', pages=xrange(1, 4), limit=4, page_count=3,
                      order=None, ordering=[], row_count=10)
            data = listlike(range(10))
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data", limit=4, max_limit=None)]
        def custom_limit(self):
            spy = Spy(var_name='data', order=None, ordering=[], row_count=10)
            data = range(10)
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data", limit=4, max_limit=7)]
        def max_limit(self):
            spy = Spy(var_name='data', order=None, ordering=[], row_count=10)
            data = range(10)
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data")]
        def default_max_pages(self):
            spy = Spy(var_name='data', limit=10, page_count=10,
                      order=None, ordering=[], row_count=100)
            data = range(100)
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data", max_pages=4)]
        def four_max_pages(self):
            spy = Spy(var_name='data', limit=10, page_count=10,
                      order=None, ordering=[], row_count=100)
            data = range(100)
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data", max_pages=3)]
        def three_max_pages(self):
            spy = Spy(var_name='data', limit=10, page_count=10,
                      order=None, ordering=[], row_count=100)
            data = range(100)
            return dict(data=data, spy=spy)

        [expose()]
        [paginate("data", limit=4, dynamic_limit='foobar')]
        def invalid_dynamic(self):
            data = range(10)
            return dict(data=data)

        [expose()]
        [paginate("data", limit=4, dynamic_limit='foobar', max_limit=None)]
        def dynamic(self):
            spy = Spy(var_name='data', pages=xrange(1, 3), limit=7, page_count=2,
                      order=None, ordering=[], row_count=10)
            data = range(10)
            return dict(data=data, spy=spy, foobar=7)

        [expose()]
        [paginate("bar")]
        [paginate("foo", limit=4)]
        def multiple(self):
            spy_foo = Spy(name='foo', var_name='foo', pages=xrange(1, 6),
                          limit=4, page_count=5, order=None, ordering=[],
                          row_count=20)
            spy_bar = Spy(name='bar', var_name='bar', pages=xrange(1, 4), limit=10,
                          page_count=3, order=None, ordering=[], row_count=26)
            foo = range(20)
            bar = list('ABCDEFGHIJKLMNOPQRSTUVWXYZ') # 26 letters
            return dict(foo=foo, bar=bar, spy_foo=spy_foo, spy_bar=spy_bar)

        [expose()]
        [paginate("data", limit=4, max_limit=None)]
        def empty(self):
            spy = Spy(var_name='data', pages=xrange(1, 1), page_count=0,
                      order=None, ordering=[], row_count=0,
                      current_page=1, first_item=0, last_item=0)
            data = []
            return dict(data=data, spy=spy)


    def test_pagination_old_style(self):
        self.request("/basic")
        assert '"data": [0, 1, 2, 3]' in self.body
        Spy.assert_ok(self.body, 'current_page', 1)
        Spy.assert_ok(self.body, 'first_item', 1)
        Spy.assert_ok(self.body, 'last_item', 4)

        self.request("/basic?tg_paginate_no=1")
        assert '"data": [0, 1, 2, 3]' in self.body
        Spy.assert_ok(self.body, 'current_page', 1)
        Spy.assert_ok(self.body, 'first_item', 1)
        Spy.assert_ok(self.body, 'last_item', 4)

        self.request("/basic?tg_paginate_no=2")
        assert '"data": [4, 5, 6, 7]' in self.body
        Spy.assert_ok(self.body, 'current_page', 2)
        Spy.assert_ok(self.body, 'first_item', 5)
        Spy.assert_ok(self.body, 'last_item', 8)

        self.request("/basic?tg_paginate_no=3")
        assert '"data": [8, 9]' in self.body
        Spy.assert_ok(self.body, 'current_page', 3)
        Spy.assert_ok(self.body, 'first_item', 9)
        Spy.assert_ok(self.body, 'last_item', 10)

    def test_pagination_new_style(self):
        self.request("/basic?data_tgp_no=1")
        assert '"data": [0, 1, 2, 3]' in self.body
        Spy.assert_ok(self.body, 'current_page', 1)
        Spy.assert_ok(self.body, 'first_item', 1)
        Spy.assert_ok(self.body, 'last_item', 4)

        self.request("/basic?data_tgp_no=2")
        assert '"data": [4, 5, 6, 7]' in self.body
        Spy.assert_ok(self.body, 'current_page', 2)
        Spy.assert_ok(self.body, 'first_item', 5)
        Spy.assert_ok(self.body, 'last_item', 8)

        self.request("/basic?data_tgp_no=3")
        assert '"data": [8, 9]' in self.body
        Spy.assert_ok(self.body, 'current_page', 3)
        Spy.assert_ok(self.body, 'first_item', 9)
        Spy.assert_ok(self.body, 'last_item', 10)

    def test_pagination_listlike(self):
        self.request("/listlike?data_tgp_no=2")
        assert '"data": [4, 5, 6, 7]' in self.body
        Spy.assert_ok(self.body, 'current_page', 2)
        Spy.assert_ok(self.body, 'first_item', 5)
        Spy.assert_ok(self.body, 'last_item', 8)

    def test_limit_override(self):
        # can't override limit (old style)
        self.request('/basic?tg_paginate_limit=2')
        assert '"data": [0, 1, 2, 3]' in self.body

        # can't override limit (new style)
        self.request("/basic?data_tgp_limit=2")
        assert '"data": [0, 1, 2, 3]' in self.body

        # can't override limit with upper bound
        self.request("/max_limit?data_tgp_limit=9")
        assert '"data": [0, 1, 2, 3, 4, 5, 6]' in self.body
        Spy.assert_ok(self.body, 'page_count', 2)
        Spy.assert_ok(self.body, 'limit', 7)
        Spy.assert_ok(self.body, 'pages', xrange(1, 3))

        # can override limit (old style)
        self.request('/custom_limit?tg_paginate_limit=2')
        assert '"data": [0, 1]' in self.body
        Spy.assert_ok(self.body, 'page_count', 5)
        Spy.assert_ok(self.body, 'limit', 2)
        Spy.assert_ok(self.body, 'pages', xrange(1, 6))

        # can override limit (new style)
        self.request("/custom_limit?data_tgp_limit=2")
        assert '"data": [0, 1]' in self.body
        Spy.assert_ok(self.body, 'page_count', 5)
        Spy.assert_ok(self.body, 'limit', 2)
        Spy.assert_ok(self.body, 'pages', xrange(1, 6))

        # can override limit with upper bound
        self.request("/max_limit?data_tgp_limit=5")
        assert '"data": [0, 1, 2, 3, 4]' in self.body
        Spy.assert_ok(self.body, 'page_count', 2)
        Spy.assert_ok(self.body, 'limit', 5)
        Spy.assert_ok(self.body, 'pages', xrange(1, 3))

    def test_limit_override_deprecated(self):
        import sys, StringIO
        stderr, sys.stderr = sys.stderr, StringIO.StringIO()
        try:
            class DeprecatedMixin(RootController):
                [expose()]
                [paginate("data", limit=4, allow_limit_override=True)]
                def custom_limit_deprecated(self):
                    spy = Spy(var_name='data',
                        order=None, ordering=[], row_count=10)
                    data = range(10)
                    return dict(data=data, spy=spy)
        finally:
            stderr, sys.stderr = sys.stderr, stderr
            warn = stderr.getvalue()
            stderr.close()
        assert 'allow_limit_override is deprecated.' in warn
        self.MyRoot.__bases__ = (DeprecatedMixin,) + self.MyRoot.__bases__
        # can override limit (using deprecated parameter)
        self.request('/custom_limit_deprecated?tg_paginate_limit=2')
        assert '"data": [0, 1]' in self.body
        Spy.assert_ok(self.body, 'page_count', 5)
        Spy.assert_ok(self.body, 'limit', 2)
        Spy.assert_ok(self.body, 'pages', xrange(1, 6))

    def test_max_pages(self):
        self.request("/default_max_pages")
        assert '"data": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in self.body
        Spy.assert_ok(self.body, 'pages', xrange(1, 6))

        self.request("/default_max_pages?data_tgp_no=5")
        assert '"data": [40, 41, 42, 43, 44, 45, 46, 47, 48, 49]' in self.body
        Spy.assert_ok(self.body, 'pages', xrange(3, 8))

        self.request("/three_max_pages")
        assert '"data": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in self.body
        Spy.assert_ok(self.body, 'pages', xrange(1, 4))

        self.request("/three_max_pages?data_tgp_no=5")
        assert '"data": [40, 41, 42, 43, 44, 45, 46, 47, 48, 49]' in self.body
        Spy.assert_ok(self.body, 'pages', xrange(4, 7))

        # even 'max pages'
        self.request("/four_max_pages")
        assert '"data": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in self.body
        Spy.assert_ok(self.body, 'pages', xrange(1, 5))

        self.request("/four_max_pages?data_tgp_no=5")
        assert '"data": [40, 41, 42, 43, 44, 45, 46, 47, 48, 49]' in self.body
        Spy.assert_ok(self.body, 'pages', xrange(4, 8))

    def test_strict_parameters(self):
        config.update({'tg.strict_parameters': True})
        try:
            # old style
            self.request("/basic?tg_paginate_no=2")
            assert self.status.startswith('200')
            assert '"data": [4, 5, 6, 7]' in self.body
            Spy.assert_ok(self.body, 'current_page', 2)
            self.request('/custom_limit?tg_paginate_limit=2')
            assert self.status.startswith('200')
            assert '"data": [0, 1]' in self.body
            Spy.assert_ok(self.body, 'limit', 2)
            self.request("/default_max_pages?tg_paginate_no=5")
            assert self.status.startswith('200')
            assert '"data": [40, 41, 42, ' in self.body
            Spy.assert_ok(self.body, 'pages', xrange(3, 8))
            self.request("/basic?tg_paginate_rubbish=2")
            assert self.status.startswith('500')
            assert ("basic() got an unexpected keyword argument"
                " 'tg_paginate_rubbish'") in self.body
            # new style
            self.request("/basic?data_tgp_no=2")
            assert self.status.startswith('200')
            assert '"data": [4, 5, 6, 7]' in self.body
            Spy.assert_ok(self.body, 'current_page', 2)
            self.request('/custom_limit?data_tgp_limit=2')
            assert self.status.startswith('200')
            assert '"data": [0, 1]' in self.body
            Spy.assert_ok(self.body, 'limit', 2)
            self.request("/default_max_pages?data_tgp_no=5")
            assert self.status.startswith('200')
            assert '"data": [40, 41, 42, ' in self.body
            Spy.assert_ok(self.body, 'pages', xrange(3, 8))
            self.request("/basic?data_tgp_rubbish=2")
            assert self.status.startswith('500')
            assert ("basic() got an unexpected keyword argument"
                " 'data_tgp_rubbish'") in self.body
        finally:
            config.update({'tg.strict_parameters': False})

    def test_invalid_dynamic_limit(self):
        self.request("/invalid_dynamic")
        assert self.status.startswith("500")
        assert 'paginate: dynamic_limit (foobar) not found in output dict' in self.body

    def test_dynamic_limit(self):
        self.request("/dynamic")
        assert '"data": [0, 1, 2, 3, 4, 5, 6]' in self.body

        self.request('/dynamic?tg_paginate_limit=2')
        assert '"data": [0, 1, 2, 3, 4, 5, 6]' in self.body

    def test_multiple(self):
        self.request("/multiple")
        assert '"foo": [0, 1, 2, 3]' in self.body
        assert '"bar": ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J"]' in self.body

        self.request("/multiple?foo_tgp_no=3")
        assert '"foo": [8, 9, 10, 11]' in self.body
        assert '"bar": ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J"]' in self.body

        self.request("/multiple?bar_tgp_no=2")
        assert '"foo": [0, 1, 2, 3]' in self.body
        assert '"bar": ["K", "L", "M", "N", "O", "P", "Q", "R", "S", "T"]' in self.body

        self.request("/multiple?foo_tgp_no=2&bar_tgp_no=3")
        assert '"foo": [4, 5, 6, 7]' in self.body
        assert '"bar": ["U", "V", "W", "X", "Y", "Z"]' in self.body

    def test_out_of_bound_pages(self):
        for number in range(-3,1):
            self.request("/basic?tg_paginate_no=%s" % number)
            assert '"data": [0, 1, 2, 3]' in self.body
            Spy.assert_ok(self.body, 'current_page', 1)
            Spy.assert_ok(self.body, 'first_item', 1)
            Spy.assert_ok(self.body, 'last_item', 4)

        for number in range(4, 7):
            self.request("/basic?tg_paginate_no=%s" % number)
            assert '"data": [8, 9]' in self.body
            Spy.assert_ok(self.body, 'current_page', 3)
            Spy.assert_ok(self.body, 'first_item', 9)
            Spy.assert_ok(self.body, 'last_item', 10)

    def test_last_page(self):
        self.request("/basic?tg_paginate_no=last")
        assert '"data": [8, 9]' in self.body
        Spy.assert_ok(self.body, 'current_page', 3)
        Spy.assert_ok(self.body, 'first_item', 9)
        Spy.assert_ok(self.body, 'last_item', 10)

    def test_href(self):
        self.request("/basic?data_tgp_no=0") # out of bound
        Spy.assert_ok(self.body, 'current_page', 1)
        Spy.assert_ok(self.body, 'href_first', None)
        Spy.assert_ok(self.body, 'href_prev', None)
        Spy.assert_ok(self.body, 'href_next', r"'/basic?data_tgp_no=2&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_last', r"'/basic?data_tgp_no=last&data_tgp_limit=4'", raw=True)

        self.request("/basic?data_tgp_no=1")
        Spy.assert_ok(self.body, 'current_page', 1)
        Spy.assert_ok(self.body, 'href_first', None)
        Spy.assert_ok(self.body, 'href_prev', None)
        Spy.assert_ok(self.body, 'href_next', r"'/basic?data_tgp_no=2&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_last', r"'/basic?data_tgp_no=last&data_tgp_limit=4'", raw=True)

        self.request("/basic?data_tgp_no=2")
        Spy.assert_ok(self.body, 'current_page', 2)
        Spy.assert_ok(self.body, 'href_first', r"'/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_prev', r"'/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_next', r"'/basic?data_tgp_no=3&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_last', r"'/basic?data_tgp_no=last&data_tgp_limit=4'", raw=True)

        self.request("/basic?data_tgp_no=3")
        Spy.assert_ok(self.body, 'current_page', 3)
        Spy.assert_ok(self.body, 'href_first', r"'/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_prev', r"'/basic?data_tgp_no=2&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_next', None)
        Spy.assert_ok(self.body, 'href_last', None)

        self.request("/basic?data_tgp_no=4") # out of bound!
        Spy.assert_ok(self.body, 'current_page', 3)
        Spy.assert_ok(self.body, 'href_first', r"'/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_prev', r"'/basic?data_tgp_no=2&data_tgp_limit=4'", raw=True)
        Spy.assert_ok(self.body, 'href_next', None)
        Spy.assert_ok(self.body, 'href_last', None)

        # empty data
        self.request("/empty")
        Spy.assert_ok(self.body, 'current_page', 1)
        Spy.assert_ok(self.body, 'href_first', None)
        Spy.assert_ok(self.body, 'href_prev', None)
        Spy.assert_ok(self.body, 'href_next', None)
        Spy.assert_ok(self.body, 'href_last', None)

    def test_with_webpath(self):
        config.update({'server.webpath': '/web/path'})
        try:
            self.request("/web/path/basic?data_tgp_no=2")
            Spy.assert_ok(self.body, 'current_page', 2)
            Spy.assert_ok(self.body, 'href_first',
                r"'/web/path/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
            Spy.assert_ok(self.body, 'href_prev',
                r"'/web/path/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
            Spy.assert_ok(self.body, 'href_next',
                r"'/web/path/basic?data_tgp_no=3&data_tgp_limit=4'", raw=True)
            Spy.assert_ok(self.body, 'href_last',
                r"'/web/path/basic?data_tgp_no=last&data_tgp_limit=4'", raw=True)
        finally:
            config.update({'server.webpath': ''})

    def test_empty_data(self):
        self.request("/empty")
        assert '"data": []' in self.body
        Spy.assert_ok(self.body, 'limit', 4)

    def test_zero_limit(self):
        self.request("/custom_limit?data_tgp_limit=0")
        assert '"data": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in self.body
        Spy.assert_ok(self.body, 'page_count', 1)
        Spy.assert_ok(self.body, 'limit', 10)
        Spy.assert_ok(self.body, 'pages', xrange(1, 2))

    def test_empty_data_zero_limit(self):
        self.request("/empty?data_tgp_limit=0")
        assert '"data": []' in self.body
        Spy.assert_ok(self.body, 'page_count', 0)
        Spy.assert_ok(self.body, 'limit', 1)


#
# Test SQLAlchemy & SQLObject
#

class Occupation(object):
    pass

class User(object):
    pass

class Address(object):
    def __repr__(self):
        # using "[...]" instead of "<...>" avoids rendering "&lt;"
        return "[Address %r]" % self.id

class SOAddress(SQLObject):
    id = IntCol(),
    user_id = IntCol()
    street = StringCol()
    city = StringCol()

    def __repr__(self):
        # using "[...]" instead of "<...>" avoids rendering "&lt;"
        return "[Address %r]" % self.id

def create_tables():
    """Create tables filled with test data."""

    occupations_table = Table('occupations', metadata,
        Column('id', Integer, primary_key=True),
        Column('name', String(20)))
    users_table = Table('users', metadata,
        Column('id', Integer, primary_key=True),
        Column('name', String(20)),
        Column('occupation_id', Integer, ForeignKey("occupations.id")))
    addresses_table = Table('addresses', metadata,
        Column('id', Integer, primary_key=True),
        Column('user_id', Integer, ForeignKey("users.id")),
        Column('street', String(50)),
        Column('city', String(40)))

    mapper(Occupation, occupations_table)
    mapper(User, users_table, properties={
        'occupation' : relation(Occupation, lazy=False),
        'addresses': relation(Address, backref='user', lazy=False)})
    mapper(Address, addresses_table)

    try:
        metadata.create_all()
    except Exception, error:
        # workaround for a problem with PySqlite < 2.6 and SQLAlchemy < 0.5
        if 'Cannot operate on a closed cursor' in str(error):
            metadata.create_all(checkfirst=False)
        else:
            raise

    occupations = (
        {'id': 1, 'name': 'Doctor'},
        {'id': 2, 'name': 'Actor'},
        {'id': 3, 'name': 'Programmer'})
    users = (
        {'id': 1, 'name': 'Smith', 'occupation_id': 1},
        {'id': 2, 'name': 'John', 'occupation_id': 2},
        {'id': 3, 'name': 'Fred', 'occupation_id': 2},
        {'id': 4, 'name': 'Albert', 'occupation_id': 3},
        {'id': 5, 'name': 'Nicole', 'occupation_id': None},
        {'id': 6, 'name': 'Sarah', 'occupation_id': None},
        {'id': 7, 'name': 'Walter', 'occupation_id': 1},
        {'id': 8, 'name': 'Bush', 'occupation_id': None});
    addresses = (
        {'id': 1, 'user_id': None, 'street': 'street P', 'city': 'city X'},
        {'id': 2, 'user_id': 2, 'street': 'street B', 'city': 'city Z'},
        {'id': 3, 'user_id': 4, 'street': 'street C', 'city': 'city X'},
        {'id': 4, 'user_id': 1, 'street': 'street D', 'city': 'city Y'},
        {'id': 5, 'user_id': None, 'street': 'street E', 'city': 'city Z'},
        {'id': 6, 'user_id': 1, 'street': 'street F', 'city': 'city Z'},
        {'id': 7, 'user_id': 3, 'street': 'street G', 'city': 'city Y'},
        {'id': 8, 'user_id': 7, 'street': 'street H', 'city': 'city Y'},
        {'id': 9, 'user_id': 1, 'street': 'street I', 'city': 'city X'},
        {'id': 10, 'user_id': 6, 'street': 'street J', 'city': 'city X'},
        {'id': 11, 'user_id': 3, 'street': 'street K', 'city': 'city Z'},
        {'id': 12, 'user_id': 2, 'street': 'street L', 'city': 'city X'},
        {'id': 13, 'user_id': 8, 'street': 'street M', 'city': 'city X'},
        {'id': 14, 'user_id': 5, 'street': 'street N', 'city': 'city Y'},
        {'id': 15, 'user_id': 1, 'street': 'street O', 'city': 'city Y'},
        {'id': 16, 'user_id': 3, 'street': 'street A', 'city': 'city X'});

    metadata.bind.execute(occupations_table.insert(), occupations)
    metadata.bind.execute(users_table.insert(), users)
    metadata.bind.execute(addresses_table.insert(), addresses)

    SOAddress.createTable()

    # SQLObject tests will need only "addresses"
    for kw in addresses:
        print SOAddress(**kw)

def drop_tables():
    metadata.drop_all()


class TestDatabasePagination(TestPagination):

    class MyRoot(RootController):

        def __common(self, method=None):
            if method == 'Q':
                data = session.query(Address)
            elif method == 'QA':
                data = session.query(Address).all()
            elif method == 'SR':
                data = SASelectResults(session.query(Address))
            elif method == 'SO':
                data = SOAddress.select()
            elif method == 'SL':
                data = list(SOAddress.select())
            else:
                raise ValueError("Invalid method %r" % method)

            spy = Spy(var_name='data', pages=xrange(1, 3), limit=10, page_count=2,
                      order=None, row_count=16)
            return dict(data=data, spy=spy)

        # default_order = basestring
        basic1 = paginate("data", default_order="id")(__common)
        basic1 = expose("turbogears.tests.paginate")(basic1)

        # default_order = list
        basic2 = paginate("data", default_order=["id"])(__common)
        basic2 = expose("turbogears.tests.paginate")(basic2)

        # default_order = basestring with default_reversed (deprecated)
        warnings.filterwarnings('ignore',
            'default_reversed is deprecated', DeprecationWarning)
        reversed1 = paginate("data", default_order="id", default_reversed=True)(__common)
        reversed1 = expose("turbogears.tests.paginate")(reversed1)

        # default_order = basestring
        reversed2 = paginate("data", default_order="-id")(__common)
        reversed2 = expose("turbogears.tests.paginate")(reversed2)

        # default_order = list with default_reversed (WRONG!)
        wrong_reversed = paginate("data", default_order=["id"], default_reversed=True)(__common)
        wrong_reversed = expose("turbogears.tests.paginate")(wrong_reversed)

        # default_order = list
        reversed3 = paginate("data", default_order=["-id"])(__common)
        reversed3 = expose("turbogears.tests.paginate")(reversed3)

        # +/+
        default_compound_ordering1 = paginate("data", default_order=["city", "street"])(__common)
        default_compound_ordering1 = expose("turbogears.tests.paginate")(default_compound_ordering1)

        # +/-
        default_compound_ordering2 = paginate("data", default_order=["city", "-street"])(__common)
        default_compound_ordering2 = expose("turbogears.tests.paginate")(default_compound_ordering2)

        # -/+
        default_compound_ordering3 = paginate("data", default_order=["-city", "street"])(__common)
        default_compound_ordering3 = expose("turbogears.tests.paginate")(default_compound_ordering3)

        # -/-
        default_compound_ordering4 = paginate("data", default_order=["-city", "-street"])(__common)
        default_compound_ordering4 = expose("turbogears.tests.paginate")(default_compound_ordering4)

        [expose("turbogears.tests.paginate")]
        [paginate("data", default_order="id")]
        def related(self):
            # only SA Query
            data = session.query(Address).outerjoin(['user', 'occupation'])

            spy = Spy(var_name='data', pages=xrange(1, 3), limit=10, page_count=2,
                      order=None, row_count=16)
            return dict(data=data, spy=spy)

        [expose("turbogears.tests.paginate")]
        [paginate("data", default_order="id", limit=0)]
        def zero_limit(self, method=None):
            if method == 'Q':
                data = session.query(Address)
            elif method == 'QA':
                data = session.query(Address).all()
            elif method == 'SR':
                data = SASelectResults(session.query(Address))
            elif method == 'SO':
                data = SOAddress.select()
            elif method == 'SL':
                data = list(SOAddress.select())
            else:
                raise Exception("Invalid method")

            spy = Spy(var_name='data', pages=xrange(1, 2), limit=16, page_count=1,
                      order=None, row_count=16)
            return dict(data=data, spy=spy)

        [expose("turbogears.tests.paginate")]
        [paginate("data", default_order="id")]
        def empty_with_groupby(self):
            data = session.query(Address)
            try:
                data = data.filter(Address.c.id < 0).group_by(Address.c.user_id)
            except AttributeError: # SQLAlchemy >= 0.5
                data = data.filter(Address.id < 0).group_by(Address.user_id)
            spy = Spy(var_name='data', pages=xrange(1, 1), limit=10, page_count=0,
                      order=None, row_count=0)
            return dict(data=data, spy=spy)


    def assert_order(self, *args):
        expr = 'data="%s"' % ''.join(['[Address %r]' % x for x in args])
        if expr not in self.body:
            print self.body
        assert expr in self.body, "expected %s" % expr

    def test_basic(self):
        for test in "basic1", "basic2":
            for method in query_methods:
                self.request("/%s?method=%s" % (test, method))
                self.assert_order(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
                Spy.assert_ok(self.body, 'current_page', 1)
                Spy.assert_ok(self.body, 'first_item', 1)
                Spy.assert_ok(self.body, 'last_item', 10)
                Spy.assert_ok(self.body, 'ordering', ['id'])

                # old style
                self.request("/%s?method=%s&tg_paginate_no=2" % (test, method))
                self.assert_order(11, 12, 13, 14, 15, 16)
                Spy.assert_ok(self.body, 'current_page', 2)
                Spy.assert_ok(self.body, 'first_item', 11)
                Spy.assert_ok(self.body, 'last_item', 16)
                Spy.assert_ok(self.body, 'ordering', ['id'])

                # new style
                self.request("/%s?method=%s&data_tgp_no=2" % (test, method))
                self.assert_order(11, 12, 13, 14, 15, 16)
                Spy.assert_ok(self.body, 'current_page', 2)
                Spy.assert_ok(self.body, 'first_item', 11)
                Spy.assert_ok(self.body, 'last_item', 16)
                Spy.assert_ok(self.body, 'ordering', ['id'])

    def test_ordering(self):
        for test in "basic1", "basic2":
            for method in query_methods:
                self.request("/%s?method=%s&tg_paginate_ordering=%s" % \
                    (test, method, 'street'))
                self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
                Spy.assert_ok(self.body, 'current_page', 1)
                Spy.assert_ok(self.body, 'first_item', 1)
                Spy.assert_ok(self.body, 'last_item', 10)
                Spy.assert_ok(self.body, 'ordering', ['street'])

                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s" % \
                    (test, method, 'street'))
                self.assert_order(11, 12, 13, 14, 15, 1)
                Spy.assert_ok(self.body, 'current_page', 2)
                Spy.assert_ok(self.body, 'first_item', 11)
                Spy.assert_ok(self.body, 'last_item', 16)
                Spy.assert_ok(self.body, 'ordering', ['street'])

    def test_strict_parameters(self):
        config.update({'tg.strict_parameters': True})
        try:
            # old style
            self.request("/basic1?method=Q&tg_paginate_ordering=street")
            self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
            Spy.assert_ok(self.body, 'ordering', ['street'])
            # new style
            self.request("/basic1?method=Q&data_tgp_ordering=street")
            assert self.status.startswith('200')
            self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
            Spy.assert_ok(self.body, 'ordering', ['street'])
        finally:
            config.update({'tg.strict_parameters': False})

    def test_default_reversed(self):
        for test in "reversed1", "reversed2", "reversed3":
            for method in query_methods:
                self.request("/%s?method=%s" % (test, method))
                self.assert_order(16, 15, 14, 13, 12, 11, 10, 9, 8, 7)
                Spy.assert_ok(self.body, 'current_page', 1)
                Spy.assert_ok(self.body, 'first_item', 1)
                Spy.assert_ok(self.body, 'last_item', 10)
                Spy.assert_ok(self.body, 'ordering', ['-id'])

                self.request("/%s?method=%s&data_tgp_no=2" % (test, method))
                self.assert_order(6, 5, 4, 3, 2, 1)
                Spy.assert_ok(self.body, 'current_page', 2)
                Spy.assert_ok(self.body, 'first_item', 11)
                Spy.assert_ok(self.body, 'last_item', 16)
                Spy.assert_ok(self.body, 'ordering', ['-id'])

    def test_invalid_default_reversed(self):
        for method in query_methods:
            self.request("/wrong_reversed?method=%s" % method)
            assert self.status.startswith("500")
            assert ('paginate: default_reversed (deprecated) only allowed'
                ' when default_order is a basestring') in self.body

    def test_reverse_ordering(self):
        for test in "basic1", "basic2":
            for method in query_methods:
                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                    % (test, method, '-street'))
                self.assert_order(1, 15, 14, 13, 12, 11, 10, 9, 8, 7)
                Spy.assert_ok(self.body, 'ordering', ['-street'])

                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                    % (test, method, '-street'))
                self.assert_order(6, 5, 4, 3, 2, 16)
                Spy.assert_ok(self.body, 'ordering', ['-street'])

    def test_compound_ordering(self):
        for test in "basic1", "basic2":
            for method in query_methods:
                # +/+
                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                    % (test, method, quote('city,street')))
                self.assert_order(16, 3, 9, 10, 12, 13, 1, 4, 7, 8)
                Spy.assert_ok(self.body, 'ordering', ['city', 'street'])

                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                    % (test, method, quote('city,street')))
                self.assert_order(14, 15, 2, 5, 6, 11)
                Spy.assert_ok(self.body, 'ordering', ['city', 'street'])

                # +/-
                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                    % (test, method, quote('city,-street')))
                self.assert_order(1, 13, 12, 10, 9, 3, 16, 15, 14, 8)
                Spy.assert_ok(self.body, 'ordering', ['city', '-street'])

                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                    % (test, method, quote('city,-street')))
                self.assert_order(7, 4, 11, 6, 5, 2)
                Spy.assert_ok(self.body, 'ordering', ['city', '-street'])

                # -/+
                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                    % (test, method, quote('-city,street')))
                self.assert_order(2, 5, 6, 11, 4, 7, 8, 14, 15, 16)
                Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])

                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                    % (test, method, quote('-city,street')))
                self.assert_order(3, 9, 10, 12, 13, 1)
                Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])

                # -/-
                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                    % (test, method, quote('-city,-street')))
                self.assert_order(11, 6, 5, 2, 15, 14, 8, 7, 4, 1)
                Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])

                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                    % (test, method, quote('-city,-street')))
                self.assert_order(13, 12, 10, 9, 3, 16)
                Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])

    def test_default_compound_ordering_1(self):
        # +/+
        for method in query_methods:
            self.request("/default_compound_ordering1?method=%s" % method)
            self.assert_order(16, 3, 9, 10, 12, 13, 1, 4, 7, 8)
            Spy.assert_ok(self.body, 'ordering', ['city', 'street'])

            self.request("/default_compound_ordering1?method=%s&tg_paginate_no=2" % method)
            self.assert_order(14, 15, 2, 5, 6, 11)
            Spy.assert_ok(self.body, 'ordering', ['city', 'street'])

    def test_default_compound_ordering_2(self):
        # +/-
        for method in query_methods:
            self.request("/default_compound_ordering2?method=%s" % method)
            self.assert_order(1, 13, 12, 10, 9, 3, 16, 15, 14, 8)
            Spy.assert_ok(self.body, 'ordering', ['city', '-street'])

            self.request("/default_compound_ordering2?method=%s&tg_paginate_no=2" % method)
            self.assert_order(7, 4, 11, 6, 5, 2)
            Spy.assert_ok(self.body, 'ordering', ['city', '-street'])

    def test_default_compound_ordering_3(self):
        # -/+
        for method in query_methods:
            self.request("/default_compound_ordering3?method=%s" % method)
            self.assert_order(2, 5, 6, 11, 4, 7, 8, 14, 15, 16)
            Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])

            self.request("/default_compound_ordering3?method=%s&tg_paginate_no=2" % method)
            self.assert_order(3, 9, 10, 12, 13, 1)
            Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])

    def test_default_compound_ordering_4(self):
        # -/-
        for method in query_methods:
            self.request("/default_compound_ordering4?method=%s" % method)
            self.assert_order(11, 6, 5, 2, 15, 14, 8, 7, 4, 1)
            Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])

            self.request("/default_compound_ordering4?method=%s&tg_paginate_no=2" % method)
            self.assert_order(13, 12, 10, 9, 3, 16)
            Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])

    def test_related_objects_ordering_level_1(self):
        # +/+
        self.request("/related?tg_paginate_ordering=%s" % quote('user.name,id'))
        self.assert_order(1, 5, 3, 13, 7, 11, 16, 2, 12, 14)
        Spy.assert_ok(self.body, 'ordering', ['user.name', 'id'])

        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
            % quote('user.name,id'))
        self.assert_order(10, 4, 6, 9, 15, 8)
        Spy.assert_ok(self.body, 'ordering', ['user.name', 'id'])

        # +/-
        self.request("/related?tg_paginate_ordering=%s" % quote('user.name,-id'))
        self.assert_order(5, 1, 3, 13, 16, 11, 7, 12, 2, 14)
        Spy.assert_ok(self.body, 'ordering', ['user.name', '-id'])

        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
            % quote('user.name,-id'))
        self.assert_order(10, 15, 9, 6, 4, 8)
        Spy.assert_ok(self.body, 'ordering', ['user.name', '-id'])

        # -/+
        self.request("/related?tg_paginate_ordering=%s" % quote('-user.name,id'))
        self.assert_order(8, 4, 6, 9, 15, 10, 14, 2, 12, 7)
        Spy.assert_ok(self.body, 'ordering', ['-user.name', 'id'])

        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
            % quote('-user.name,id'))
        self.assert_order(11, 16, 13, 3, 1, 5)
        Spy.assert_ok(self.body, 'ordering', ['-user.name', 'id'])

        # -/-
        self.request("/related?tg_paginate_ordering=%s" % quote('-user.name,-id'))
        self.assert_order(8, 15, 9, 6, 4, 10, 14, 12, 2, 16)
        Spy.assert_ok(self.body, 'ordering', ['-user.name', '-id'])

        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
            % quote('-user.name,-id'))
        self.assert_order(11, 7, 13, 3, 5, 1)
        Spy.assert_ok(self.body, 'ordering', ['-user.name', '-id'])

    def test_related_objects_ordering_level_2(self):
        # +/+
        self.request("/related?tg_paginate_ordering=%s"
            % quote('user.occupation.name,id'))
        self.assert_order(1, 5, 10, 13, 14, 2, 7, 11, 12, 16)
        Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', 'id'])

        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
            % quote('user.occupation.name,id'))
        self.assert_order(4, 6, 8, 9, 15, 3)
        Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', 'id'])

        # +/-
        self.request("/related?tg_paginate_ordering=%s"
            % quote('user.occupation.name,-id'))
        self.assert_order(14, 13, 10, 5, 1, 16, 12, 11, 7, 2)
        Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', '-id'])

        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
            % quote('user.occupation.name,-id'))
        self.assert_order(15, 9, 8, 6, 4, 3)
        Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', '-id'])

        # -/+
        self.request("/related?tg_paginate_ordering=%s"
            % quote('-user.occupation.name,id'))
        self.assert_order(3, 4, 6, 8, 9, 15, 2, 7, 11, 12)
        Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', 'id'])

        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
            % quote('-user.occupation.name,id'))
        self.assert_order(16, 1, 5, 10, 13, 14)
        Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', 'id'])

        # -/-
        self.request("/related?tg_paginate_ordering=%s"
            % quote('-user.occupation.name,-id'))
        self.assert_order(3, 15, 9, 8, 6, 4, 16, 12, 11, 7)
        Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', '-id'])

        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"#
            % quote('-user.occupation.name,-id'))
        self.assert_order(2, 14, 13, 10, 5, 1)
        Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', '-id'])

    def test_zero_limit(self):
        for method in query_methods:
            self.request("/zero_limit?method=%s" % method)
            self.assert_order(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)

    def test_ticket_1641(self):
        self.request("/empty_with_groupby")
        self.assert_order()


def setup_module():
    global _sa_dburi, _so_dburi
    _so_dburi = config.get("sqlobject.dburi", "sqlite:///:memory:")
    _sa_dburi = config.get("sqlalchemy.dburi", "sqlite:///:memory:")
    # sqlalchemy setup
    database.set_db_uri({"sqlalchemy.dburi": "sqlite:///:memory:"})
    sqlalchemy_cleanup()
    get_engine()
    # sqlobject setup
    database.set_db_uri("sqlite:///:memory:")
    # table setup
    create_tables()
    hub.commit()

def teardown_module():
    drop_tables()
    sqlalchemy_cleanup()
    config.update({"sqlalchemy.dburi": _sa_dburi, 'sqlobject.dburi': _so_dburi})